@@ -12,7 +12,14 @@ import types, sys, string, datetime, logging, time
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmPG2, gmExceptions, gmNull, gmBusinessDBObject, gmDateTime, gmTools, gmI18N
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmBusinessDBObject
+from Gnumed.pycommon import gmNull
+from Gnumed.pycommon import gmExceptions
+
 from Gnumed.business import gmClinNarrative
 
 
@@ -42,7 +49,7 @@ def diagnostic_certainty_classification2str(classification):
 	try:
 		return __diagnostic_certainty_classification_map[classification]
 	except KeyError:
-		return _(u'%s: unknown diagnostic certainty classification') % classification
+		return _(u'<%s>: unknown diagnostic certainty classification') % classification
 #============================================================
 # Health Issues API
 #============================================================
@@ -63,6 +70,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 	_cmds_store_payload = [
 		u"""update clin.health_issue set
 				description = %(description)s,
+				summary = gm.nullify_empty_string(%(summary)s),
 				age_noted = %(age_noted)s,
 				laterality = gm.nullify_empty_string(%(laterality)s),
 				grouping = gm.nullify_empty_string(%(grouping)s),
@@ -78,6 +86,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 	]
 	_updatable_fields = [
 		'description',
+		'summary',
 		'grouping',
 		'age_noted',
 		'laterality',
@@ -119,6 +128,8 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 
 		gmBusinessDBObject.cBusinessDBObject.__init__(self, row=r)
 	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
 	def rename(self, description=None):
 		"""Method for issue renaming.
 
@@ -143,7 +154,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		return True
 	#--------------------------------------------------------
 	def get_episodes(self):
-		cmd = u"select * from clin.v_pat_episodes where pk_health_issue = %(pk)s"
+		cmd = u"SELECT * FROM clin.v_pat_episodes WHERE pk_health_issue = %(pk)s"
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'pk': self.pk_obj}}], get_col_idx = True)
 		return [ cEpisode(row = {'data': r, 'idx': idx, 'pk_field': 'pk_episode'})  for r in rows ]
 	#--------------------------------------------------------
@@ -192,18 +203,58 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		# later we can improve this deeper inside
 		return gmDateTime.format_interval_medically(self._payload[self._idx['age_noted']])
 	#--------------------------------------------------------
-	def _get_laterality_description(self):
-		try:
-			return laterality2str[self._payload[self._idx['laterality']]]
-		except KeyError:
-			return u'<???>'
+	def format_as_journal(self, left_margin=0, date_format='%a, %b %d %Y'):
+		rows = gmClinNarrative.get_as_journal (
+			issues = (self.pk_obj,),
+			order_by = u'pk_episode, pk_encounter, clin_when, scr, src_table'
+		)
 
-	laterality_description = property(_get_laterality_description, lambda x:x)
-	#--------------------------------------------------------
-	def _get_diagnostic_certainty_description(self):
-		return diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']])
+		if len(rows) == 0:
+			return u''
 
-	diagnostic_certainty_description = property(_get_diagnostic_certainty_description, lambda x:x)
+		left_margin = u' ' * left_margin
+
+		lines = []
+		lines.append(_('Clinical data generated during encounters under this health issue:'))
+
+		prev_epi = None
+		for row in rows:
+			if row['pk_episode'] != prev_epi:
+				lines.append(u'')
+				prev_epi = row['pk_episode']
+
+			when = row['clin_when'].strftime(date_format).decode(gmI18N.get_encoding())
+			top_row = u'%s%s %s (%s) %s' % (
+				gmTools.u_box_top_left_arc,
+				gmTools.u_box_horiz_single,
+				gmClinNarrative.soap_cat2l10n_str[row['real_soap_cat']],
+				when,
+				gmTools.u_box_horiz_single * 5
+			)
+			soap = gmTools.wrap (
+				text = row['narrative'],
+				width = 60,
+				initial_indent = u'  ',
+				subsequent_indent = u'  ' + left_margin
+			)
+			row_ver = u''
+			if row['row_version'] > 0:
+				row_ver = u'v%s: ' % row['row_version']
+			bottom_row = u'%s%s %s, %s%s %s' % (
+				u' ' * 40,
+				gmTools.u_box_horiz_light_heavy,
+				row['modified_by'],
+				row_ver,
+				row['date_modified'],
+				gmTools.u_box_horiz_heavy_light
+			)
+
+			lines.append(top_row)
+			lines.append(soap)
+			lines.append(bottom_row)
+
+		eol_w_margin = u'\n%s' % left_margin
+		return left_margin + eol_w_margin.join(lines) + u'\n'
 	#--------------------------------------------------------
 	def format(self, left_margin=0, patient=None):
 
@@ -251,7 +302,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		if self._payload[self._idx['age_noted']] is not None:
 			lines.append(_(' Noted at age: %s') % self.age_noted_human_readable())
 
-		lines.append(_(' Status: %s, %s%s') % (
+		lines.append(u' ' + _('Status') + u': %s, %s%s' % (
 			gmTools.bool2subst(self._payload[self._idx['is_active']], _('active'), _('inactive')),
 			gmTools.bool2subst(self._payload[self._idx['clinically_relevant']], _('clinically relevant'), _('not clinically relevant')),
 			gmTools.coalesce (
@@ -259,9 +310,19 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 				instead = u'',
 				template_initial = u', %s',
 				none_equivalents = [None, u'']
-			),
+			)
 		))
-		lines.append('')
+
+		if self._payload[self._idx['summary']] is not None:
+			lines.append(u'')
+			lines.append(gmTools.wrap (
+				text = self._payload[self._idx['summary']],
+				width = 60,
+				initial_indent = u'  ',
+				subsequent_indent = u'  '
+			))
+
+		lines.append(u'')
 
 		emr = patient.get_emr()
 
@@ -280,7 +341,6 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 					gmTools.u_right_double_angle_quote
 				)
 			)
-			lines.append('')
 			for epi in epis:
 				lines.append(u' \u00BB%s\u00AB (%s)' % (
 					epi['description'],
@@ -377,6 +437,39 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		left_margin = u' ' * left_margin
 		eol_w_margin = u'\n%s' % left_margin
 		return left_margin + eol_w_margin.join(lines) + u'\n'
+	#--------------------------------------------------------
+	# properties
+	#--------------------------------------------------------
+	episodes = property(get_episodes, lambda x:x)
+	#--------------------------------------------------------
+	open_episode = property(get_open_episode, lambda x:x)
+	#--------------------------------------------------------
+	def _get_latest_episode(self):
+		cmd = u"""SELECT
+			coalesce (
+				(SELECT pk FROM clin.episode WHERE fk_health_issue = %(issue)s AND is_open IS TRUE),
+				(SELECT pk FROM clin.v_pat_episodes WHERE fk_health_issue = %(issue)s ORDER BY last_affirmed DESC limit 1)
+		)"""
+		args = {'issue': self.pk_obj}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		if len(rows) == 0:
+			return None
+		return cEpisode(aPK_obj = rows[0][0])
+
+	latest_episode = property(_get_latest_episode, lambda x:x)
+	#--------------------------------------------------------
+	def _get_laterality_description(self):
+		try:
+			return laterality2str[self._payload[self._idx['laterality']]]
+		except KeyError:
+			return u'<???>'
+
+	laterality_description = property(_get_laterality_description, lambda x:x)
+	#--------------------------------------------------------
+	def _get_diagnostic_certainty_description(self):
+		return diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']])
+
+	diagnostic_certainty_description = property(_get_diagnostic_certainty_description, lambda x:x)
 #============================================================
 def create_health_issue(description=None, encounter=None, patient=None):
 	"""Creates a new health issue for a given patient.
@@ -425,7 +518,8 @@ def get_dummy_health_issue():
 		'clinically_relevant': True,
 		'is_confidential': None,
 		'is_cause_of_death': False,
-		'is_dummy': True
+		'is_dummy': True,
+		'grouping': None
 	}
 	return issue
 #-----------------------------------------------------------
@@ -450,6 +544,7 @@ class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 				fk_health_issue = %(pk_health_issue)s,
 				is_open = %(episode_open)s::boolean,
 				description = %(description)s,
+				summary = gm.nullify_empty_string(%(summary)s),
 				diagnostic_certainty_classification = gm.nullify_empty_string(%(diagnostic_certainty_classification)s)
 			where
 				pk = %(pk_episode)s and
@@ -460,6 +555,7 @@ class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 		'pk_health_issue',
 		'episode_open',
 		'description',
+		'summary',
 		'diagnostic_certainty_classification'
 	]
 	#--------------------------------------------------------
@@ -501,6 +597,8 @@ class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 		else:
 			gmBusinessDBObject.cBusinessDBObject.__init__(self, aPK_obj=pk, row=row)
 	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
 	def get_access_range(self):
 		"""Get earliest and latest access to this episode.
 
@@ -544,6 +642,9 @@ from (
 	def get_patient(self):
 		return self._payload[self._idx['pk_patient']]
 	#--------------------------------------------------------
+	def get_narrative(self, soap_cats=None, encounters=None):
+		return gmClinNarrative.get_narrative(soap_cats = None, encounters = encounters, episodes = [self.pk_obj])
+	#--------------------------------------------------------
 	def rename(self, description=None):
 		"""Method for episode editing, that is, episode renaming.
 
@@ -567,10 +668,60 @@ from (
 			return False
 		return True
 	#--------------------------------------------------------
-	def _get_diagnostic_certainty_description(self):
-		return diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']])
+	def format_as_journal(self, left_margin=0, date_format='%a, %b %d %Y'):
+		rows = gmClinNarrative.get_as_journal (
+			episodes = (self.pk_obj,),
+			order_by = u'pk_encounter, clin_when, scr, src_table'
+			#order_by = u'pk_encounter, scr, clin_when, src_table'
+		)
 
-	diagnostic_certainty_description = property(_get_diagnostic_certainty_description, lambda x:x)
+		if len(rows) == 0:
+			return u''
+
+		lines = []
+
+		lines.append(_('Clinical data generated during encounters within this episode:'))
+
+		left_margin = u' ' * left_margin
+
+		prev_enc = None
+		for row in rows:
+			if row['pk_encounter'] != prev_enc:
+				lines.append(u'')
+				prev_enc = row['pk_encounter']
+
+			when = row['clin_when'].strftime(date_format).decode(gmI18N.get_encoding())
+			top_row = u'%s%s %s (%s) %s' % (
+				gmTools.u_box_top_left_arc,
+				gmTools.u_box_horiz_single,
+				gmClinNarrative.soap_cat2l10n_str[row['real_soap_cat']],
+				when,
+				gmTools.u_box_horiz_single * 5
+			)
+			soap = gmTools.wrap (
+				text = row['narrative'],
+				width = 60,
+				initial_indent = u'  ',
+				subsequent_indent = u'  ' + left_margin
+			)
+			row_ver = u''
+			if row['row_version'] > 0:
+				row_ver = u'v%s: ' % row['row_version']
+			bottom_row = u'%s%s %s, %s%s %s' % (
+				u' ' * 40,
+				gmTools.u_box_horiz_light_heavy,
+				row['modified_by'],
+				row_ver,
+				row['date_modified'],
+				gmTools.u_box_horiz_heavy_light
+			)
+
+			lines.append(top_row)
+			lines.append(soap)
+			lines.append(bottom_row)
+
+		eol_w_margin = u'\n%s' % left_margin
+		return left_margin + eol_w_margin.join(lines) + u'\n'
 	#--------------------------------------------------------
 	def format(self, left_margin=0, patient=None):
 
@@ -585,44 +736,76 @@ from (
 		lines = []
 
 		# episode details
-		lines.append (_('Episode %s%s%s (%s%s)   [#%s]\n') % (
+		lines.append (_('Episode %s%s%s   [#%s]') % (
 			gmTools.u_left_double_angle_quote,
 			self._payload[self._idx['description']],
 			gmTools.u_right_double_angle_quote,
-			gmTools.coalesce (
-				initial = diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']]),
-				instead = u'',
-				template_initial = u'%s, ',
-				none_equivalents = [None, u'']
-			),
-			gmTools.bool2subst(self._payload[self._idx['episode_open']], _('active'), _('finished')),
 			self._payload[self._idx['pk_episode']]
 		))
 
 		enc = cEncounter(aPK_obj = self._payload[self._idx['pk_encounter']])
-		lines.append (_('Created during encounter: %s (%s - %s)   [#%s]\n') % (
+		lines.append (u' ' + _('Created during encounter: %s (%s - %s)   [#%s]') % (
 			enc['l10n_type'],
 			enc['started_original_tz'].strftime('%Y-%m-%d %H:%M'),
 			enc['last_affirmed_original_tz'].strftime('%H:%M'),
 			self._payload[self._idx['pk_encounter']]
 		))
 
-		# encounters
 		emr = patient.get_emr()
 		encs = emr.get_encounters(episodes = [self._payload[self._idx['pk_episode']]])
 		first_encounter = None
 		last_encounter = None
+		if (encs is not None) and (len(encs) > 0):
+			first_encounter = emr.get_first_encounter(episode_id = self._payload[self._idx['pk_episode']])
+			last_encounter = emr.get_last_encounter(episode_id = self._payload[self._idx['pk_episode']])
+			if self._payload[self._idx['episode_open']]:
+				end = gmDateTime.pydt_now_here()
+				end_str = gmTools.u_ellipsis
+			else:
+				end = last_encounter['last_affirmed']
+				end_str = last_encounter['last_affirmed'].strftime('%m/%Y')
+			age = gmDateTime.format_interval_medically(end - first_encounter['started'])
+			lines.append(_(' Duration: %s (%s - %s)') % (
+				age,
+				first_encounter['started'].strftime('%m/%Y'),
+				end_str
+			))
+
+		lines.append(u' ' + _('Status') + u': %s%s' % (
+			gmTools.bool2subst(self._payload[self._idx['episode_open']], _('active'), _('finished')),
+			gmTools.coalesce (
+				initial = diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']]),
+				instead = u'',
+				template_initial = u', %s',
+				none_equivalents = [None, u'']
+			)
+		))
+
+		if self._payload[self._idx['summary']] is not None:
+			lines.append(u'')
+			lines.append(gmTools.wrap (
+					text = self._payload[self._idx['summary']],
+					width = 60,
+					initial_indent = u'  ',
+					subsequent_indent = u'  '
+				)
+			)
+
+		lines.append(u'')
+
+		# encounters
 		if encs is None:
 			lines.append(_('Error retrieving encounters for this episode.'))
 		elif len(encs) == 0:
 			lines.append(_('There are no encounters for this episode.'))
 		else:
-			first_encounter = emr.get_first_encounter(episode_id = self._payload[self._idx['pk_episode']])
-			last_encounter = emr.get_last_encounter(episode_id = self._payload[self._idx['pk_episode']])
-
 			lines.append(_('Last worked on: %s\n') % last_encounter['last_affirmed_original_tz'].strftime('%Y-%m-%d %H:%M'))
 
-			lines.append(_('1st and (up to 3) most recent (of %s) encounters (%s - %s):') % (
+			if len(encs) < 4:
+				line = _('%s encounter(s) (%s - %s):')
+			else:
+				line = _('1st and (up to 3) most recent (of %s) encounters (%s - %s):')
+			lines.append(line % (
 				len(encs),
 				first_encounter['started'].strftime('%m/%Y'),
 				last_encounter['last_affirmed'].strftime('%m/%Y')
@@ -752,6 +935,32 @@ from (
 		left_margin = u' ' * left_margin
 		eol_w_margin = u'\n%s' % left_margin
 		return left_margin + eol_w_margin.join(lines) + u'\n'
+	#--------------------------------------------------------
+	# properties
+	#--------------------------------------------------------
+	def _get_diagnostic_certainty_description(self):
+		return diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']])
+
+	diagnostic_certainty_description = property(_get_diagnostic_certainty_description, lambda x:x)
+	#--------------------------------------------------------
+	def _get_has_narrative(self):
+		cmd = u"""SELECT EXISTS (
+			SELECT 1 FROM clin.clin_narrative
+			WHERE
+				fk_episode = %(epi)s
+					AND
+				fk_encounter IN (
+					SELECT pk FROM clin.encounter WHERE fk_patient = %(pat)s
+				)
+		)"""
+		args = {
+			u'pat': self._payload[self._idx['pk_patient']],
+			u'epi': self._payload[self._idx['pk_episode']]
+		}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		return rows[0][0]
+
+	has_narrative = property(_get_has_narrative, lambda x:x)
 #============================================================
 def create_episode(pk_health_issue=None, episode_name=None, is_open=False, allow_dupes=False, encounter=None):
 	"""Creates a new episode for a given patient's health issue.
@@ -805,6 +1014,7 @@ def episode2problem(episode=None, allow_closed=False):
 #============================================================
 class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one encounter."""
+
 	_cmd_fetch_payload = u"select * from clin.v_pat_encounters where pk_encounter = %s"
 	_cmds_store_payload = [
 		u"""update clin.encounter set
@@ -994,6 +1204,34 @@ limit 1
 
 		return rows[0][0]
 	#--------------------------------------------------------
+	def get_episodes(self, exclude=None):
+		cmd = u"""
+SELECT * FROM clin.v_pat_episodes
+WHERE
+	pk_episode IN (
+
+		SELECT DISTINCT fk_episode
+		FROM clin.clin_root_item
+		WHERE fk_encounter = %%(enc)s
+
+			UNION
+
+		SELECT DISTINCT fk_episode
+		FROM blobs.doc_med
+		WHERE fk_encounter = %%(enc)s
+	)
+	%s"""
+		args = {'enc': self.pk_obj}
+		if exclude is not None:
+			cmd = cmd % u'AND pk_episode NOT IN %(excluded)s'
+			args['excluded'] = tuple(exclude)
+		else:
+			cmd = cmd % u''
+
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+
+		return [ cEpisode(row = {'data': r, 'idx': idx, 'pk_field': 'pk_episode'})  for r in rows ]
+	#--------------------------------------------------------
 	def format_soap(self, episodes=None, left_margin=0, soap_cats='soap', emr=None, issues=None):
 
 		lines = []
@@ -1026,8 +1264,70 @@ limit 1
 
 		return lines
 	#--------------------------------------------------------
-	def format(self, episodes=None, with_soap=False, left_margin=0, patient=None, issues=None, with_docs=True, with_tests=True, fancy_header=True, with_vaccinations=True):
+	def format_latex(self, date_format=None, soap_cats=None):
 
+		if date_format is None:
+			date_format = '%A, %B %d %Y'
+
+		tex = u'\\multicolumn{2}{l}{%s: %s ({\\footnotesize %s - %s})} \\tabularnewline \n' % (
+			gmTools.tex_escape_string(self._payload[self._idx['l10n_type']]),
+			self._payload[self._idx['started']].strftime(date_format).decode(gmI18N.get_encoding()),
+			self._payload[self._idx['started']].strftime('%H:%M'),
+			self._payload[self._idx['last_affirmed']].strftime('%H:%M')
+		)
+		tex += u'\\hline \\tabularnewline \n'
+
+		for epi in self.get_episodes():
+			soaps = epi.get_narrative(soap_cats = soap_cats, encounters = [self.pk_obj])
+			if len(soaps) == 0:
+				continue
+			tex += u'\\multicolumn{2}{l}{\\emph{%s%s%s%s}} \\tabularnewline \n' % (
+#			tex += u' & \\emph{%s%s%s%s} \\tabularnewline \n' % (
+				gmTools.tex_escape_string(epi['description']),
+				gmTools.coalesce (
+					initial = diagnostic_certainty_classification2str(epi['diagnostic_certainty_classification']),
+					instead = u'',
+					template_initial = u' {\\footnotesize [%s]}',
+					none_equivalents = [None, u'']
+				),
+				gmTools.tex_escape_string(gmTools.coalesce(epi['health_issue'], u'', u' (%s)')),
+				gmTools.coalesce (
+					initial = diagnostic_certainty_classification2str(epi['diagnostic_certainty_classification_issue']),
+					instead = u'',
+					template_initial = u' {\\footnotesize [%s]}',
+					none_equivalents = [None, u'']
+				),
+			)
+			for soap in soaps:
+				tex += u'{\\small %s} & {\\small %s} \\tabularnewline \n' % (
+					gmClinNarrative.soap_cat2l10n[soap['soap_cat']],
+					gmTools.tex_escape_string(soap['narrative'].strip(u'\n'))
+				)
+			tex += u' & \\tabularnewline \n'
+
+		if self._payload[self._idx['reason_for_encounter']] is not None:
+			tex += u'%s & %s \\tabularnewline \n' % (
+				gmTools.tex_escape_string(_('RFE')),
+				gmTools.tex_escape_string(self._payload[self._idx['reason_for_encounter']])
+			)
+		if self._payload[self._idx['assessment_of_encounter']] is not None:
+			tex += u'%s & %s \\tabularnewline \n' % (
+				gmTools.tex_escape_string(_('AOE')),
+				gmTools.tex_escape_string(self._payload[self._idx['assessment_of_encounter']])
+			)
+
+		tex += u'\\hline \\tabularnewline \n'
+		tex += u' & \\tabularnewline \n'
+
+		return tex
+	#--------------------------------------------------------
+	def format(self, episodes=None, with_soap=False, left_margin=0, patient=None, issues=None, with_docs=True, with_tests=True, fancy_header=True, with_vaccinations=True, with_co_encountlet_hints=False, with_rfe_aoe=False):
+		"""Format an encounter.
+
+		with_co_encountlet_hints:
+			- whether to include which *other* episodes were discussed during this encounter
+			- (only makes sense if episodes != None)
+		"""
 		lines = []
 
 		if fancy_header:
@@ -1053,14 +1353,11 @@ limit 1
 				gmTools.bool2subst(gmDateTime.dst_currently_in_effect, u' - ' + _('daylight savings time in effect'), u'')
 			))
 
-			lines.append(u'%s: %s' % (
-				_('RFE'),
-				gmTools.coalesce(self._payload[self._idx['reason_for_encounter']], u'')
-			))
-			lines.append(u'%s: %s' % (
-				_('AOE'),
-				gmTools.coalesce(self._payload[self._idx['assessment_of_encounter']], u'')
-			))
+			if self._payload[self._idx['reason_for_encounter']] is not None:
+				lines.append(u'%s: %s' % (_('RFE'), self._payload[self._idx['reason_for_encounter']]))
+
+			if self._payload[self._idx['assessment_of_encounter']] is not None:
+				lines.append(u'%s: %s' % (_('AOE'), self._payload[self._idx['assessment_of_encounter']]))
 
 		else:
 			lines.append(u'%s%s: %s - %s%s' % (
@@ -1070,6 +1367,11 @@ limit 1
 				self._payload[self._idx['last_affirmed_original_tz']].strftime('%H:%M'),
 				gmTools.coalesce(self._payload[self._idx['assessment_of_encounter']], u'', u' \u00BB%s\u00AB')
 			))
+			if with_rfe_aoe:
+				if self._payload[self._idx['reason_for_encounter']] is not None:
+					lines.append(u'%s: %s' % (_('RFE'), self._payload[self._idx['reason_for_encounter']]))
+				if self._payload[self._idx['assessment_of_encounter']] is not None:
+					lines.append(u'%s: %s' % (_('AOE'), self._payload[self._idx['assessment_of_encounter']]))
 
 		if with_soap:
 			lines.append(u'')
@@ -1136,7 +1438,7 @@ limit 1
 			)
 
 			if len(docs) > 0:
-				lines.append('')
+				lines.append(u'')
 				lines.append(_('Documents:'))
 
 			for d in docs:
@@ -1149,6 +1451,21 @@ limit 1
 
 			del docs
 
+		# co-encountlets
+		if with_co_encountlet_hints:
+			if episodes is not None:
+				other_epis = self.get_episodes(exclude = episodes)
+				if len(other_epis) > 0:
+					lines.append(u'')
+					lines.append(_('%s other episodes touched upon during this encounter:') % len(other_epis))
+				for epi in other_epis:
+					lines.append(u' %s%s%s%s' % (
+						gmTools.u_left_double_angle_quote,
+						epi['description'],
+						gmTools.u_right_double_angle_quote,
+						gmTools.coalesce(epi['health_issue'], u'', u' (%s)')
+					))
+
 		eol_w_margin = u'\n%s' % (u' ' * left_margin)
 		return u'%s\n' % eol_w_margin.join(lines)
 #-----------------------------------------------------------
@@ -1322,11 +1639,11 @@ class cProblem(gmBusinessDBObject.cBusinessDBObject):
 			gmBusinessDBObject.cBusinessDBObject.__init__(self, aPK_obj=pk)
 			return
 		except gmExceptions.ConstructorError:
-			_log.exception('problem not found, trying potential problems')
+			_log.exception('actual problem not found, trying "potential" problems')
 			if try_potential_problems is False:
 				raise
 
-		# try to instantiate from non-problem view
+		# try to instantiate from potential-problems view
 		cProblem._cmd_fetch_payload = u"""
 			SELECT *, True as is_potential_problem
 			FROM clin.v_potential_problem_list
@@ -1341,13 +1658,91 @@ class cProblem(gmBusinessDBObject.cBusinessDBObject):
 		if self._payload[self._idx['type']] != 'episode':
 			_log.error('cannot convert problem [%s] of type [%s] to episode' % (self._payload[self._idx['problem']], self._payload[self._idx['type']]))
 			return None
-		return cEpisode(aPK_obj=self._payload[self._idx['pk_episode']])
+		return cEpisode(aPK_obj = self._payload[self._idx['pk_episode']])
+	#--------------------------------------------------------
+	def get_visual_progress_notes(self, encounter_id=None):
+
+		if self._payload[self._idx['type']] == u'issue':
+			episodes = [ cHealthIssue(aPK_obj = self._payload[self._idx['pk_health_issue']]).latest_episode ]
+			#xxxxxxxxxxxxx
+
+		emr = patient.get_emr()
+
+		doc_folder = gmDocuments.cDocumentFolder(aPKey = patient.ID)
+		return doc_folder.get_visual_progress_notes (
+			health_issue = self._payload[self._idx['pk_health_issue']],
+			episode = self._payload[self._idx['pk_episode']]
+		)
+	#--------------------------------------------------------
+	# properties
 	#--------------------------------------------------------
 	# doubles as 'diagnostic_certainty_description' getter:
 	def get_diagnostic_certainty_description(self):
 		return diagnostic_certainty_classification2str(self._payload[self._idx['diagnostic_certainty_classification']])
 
 	diagnostic_certainty_description = property(get_diagnostic_certainty_description, lambda x:x)
+#-----------------------------------------------------------
+def problem2episode(problem=None):
+	"""Retrieve the cEpisode instance equivalent to the given problem.
+
+	The problem's type attribute must be 'episode'
+
+	@param problem: The problem to retrieve its related episode for
+	@type problem: A gmEMRStructItems.cProblem instance
+	"""
+	if isinstance(problem, cEpisode):
+		return problem
+
+	exc = TypeError('cannot convert [%s] to episode' % problem)
+
+	if not isinstance(problem, cProblem):
+		raise exc
+
+	if problem['type'] != 'episode':
+		raise exc
+
+	return cEpisode(aPK_obj = problem['pk_episode'])
+#-----------------------------------------------------------
+def problem2issue(problem=None):
+	"""Retrieve the cIssue instance equivalent to the given problem.
+
+	The problem's type attribute must be 'issue'.
+
+	@param problem: The problem to retrieve the corresponding issue for
+	@type problem: A gmEMRStructItems.cProblem instance
+	"""
+	if isinstance(problem, cHealthIssue):
+		return problem
+
+	exc = TypeError('cannot convert [%s] to health issue' % problem)
+
+	if not isinstance(problem, cProblem):
+		raise exc
+
+	if  problem['type'] != 'issue':
+		raise exc
+
+	return cHealthIssue(aPK_obj = problem['pk_health_issue'])
+#-----------------------------------------------------------
+def reclass_problem(self, problem=None):
+	"""Transform given problem into either episode or health issue instance.
+	"""
+	if isinstance(problem, (cEpisode, cHealthIssue)):
+		return problem
+
+	exc = TypeError('cannot reclass [%s] instance to either episode or health issue' % type(problem))
+
+	if not isinstance(problem, cProblem):
+		_log.debug(u'%s' % problem)
+		raise exc
+
+	if problem['type'] == 'episode':
+		return cEpisode(aPK_obj = problem['pk_episode'])
+
+	if problem['type'] == 'issue':
+		return cHealthIssue(aPK_obj = problem['pk_health_issue'])
+
+	raise exc
 #============================================================
 class cHospitalStay(gmBusinessDBObject.cBusinessDBObject):
 
@@ -1393,12 +1788,10 @@ class cHospitalStay(gmBusinessDBObject.cBusinessDBObject):
 #-----------------------------------------------------------
 def get_patient_hospital_stays(patient=None):
 
-	queries = [
-		{
-		'cmd': u'select * from clin.v_pat_hospital_stays where pk_patient = %(pat)s order by admission',
+	queries = [{
+		'cmd': u'SELECT * FROM clin.v_pat_hospital_stays WHERE pk_patient = %(pat)s ORDER BY admission',
 		'args': {'pat': patient}
-		}
-	]
+	}]
 
 	rows, idx = gmPG2.run_ro_queries(queries = queries, get_col_idx = True)
 
@@ -1406,20 +1799,16 @@ def get_patient_hospital_stays(patient=None):
 #-----------------------------------------------------------
 def create_hospital_stay(encounter=None, episode=None):
 
-	queries = [
-		{
-		 'cmd': u'insert into clin.hospital_stay (fk_encounter, fk_episode) values (%(enc)s, %(epi)s)',
+	queries = [{
+		 'cmd': u'INSERT INTO clin.hospital_stay (fk_encounter, fk_episode) VALUES (%(enc)s, %(epi)s) RETURNING pk',
 		 'args': {'enc': encounter, 'epi': episode}
-		},
-		{'cmd': u"select currval('clin.hospital_stay_pk_seq')"}
-	]
-
+	}]
 	rows, idx = gmPG2.run_rw_queries(queries = queries, return_data = True)
 
 	return cHospitalStay(aPK_obj = rows[0][0])
 #-----------------------------------------------------------
 def delete_hospital_stay(stay=None):
-	cmd = u'delete from clin.hospital_stay where pk = %(pk)s'
+	cmd = u'DELETE FROM clin.hospital_stay WHERE pk = %(pk)s'
 	args = {'pk': stay}
 	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 	return True
@@ -1429,7 +1818,10 @@ class cPerformedProcedure(gmBusinessDBObject.cBusinessDBObject):
 	_cmd_fetch_payload = u"select * from clin.v_pat_procedures where pk_procedure = %s"
 	_cmds_store_payload = [
 		u"""UPDATE clin.procedure SET
+				soap_cat = 'p',
 				clin_when = %(clin_when)s,
+				clin_end = %(clin_end)s,
+				is_ongoing = %(is_ongoing)s,
 				clin_where = NULLIF (
 					COALESCE (
 						%(pk_hospital_stay)s::TEXT,
@@ -1445,10 +1837,11 @@ class cPerformedProcedure(gmBusinessDBObject.cBusinessDBObject):
 				pk = %(pk_procedure)s AND
 				xmin = %(xmin_procedure)s
 			RETURNING xmin as xmin_procedure"""
-#		,u"""select xmin_procedure from clin.v_pat_procedures where pk_procedure = %(pk_procedure)s"""
 	]
 	_updatable_fields = [
 		'clin_when',
+		'clin_end',
+		'is_ongoing',
 		'clin_where',
 		'performed_procedure',
 		'pk_hospital_stay',
@@ -1468,9 +1861,19 @@ class cPerformedProcedure(gmBusinessDBObject.cBusinessDBObject):
 	#-------------------------------------------------------
 	def format(self, left_margin=0, include_episode=True):
 
-		line = u'%s%s (%s): %s' % (
+		if self._payload[self._idx['is_ongoing']]:
+			end = _(' (ongoing)')
+		else:
+			end = self._payload[self._idx['clin_end']]
+			if end is None:
+				end = u''
+			else:
+				end = u' - %s' % end.strftime('%Y %b %d').decode(gmI18N.get_encoding())
+
+		line = u'%s%s%s, %s: %s' % (
 			(u' ' * left_margin),
 			self._payload[self._idx['clin_when']].strftime('%Y %b %d').decode(gmI18N.get_encoding()),
+			end,
 			self._payload[self._idx['clin_where']],
 			self._payload[self._idx['performed_procedure']]
 		)
@@ -1499,12 +1902,14 @@ def create_performed_procedure(encounter=None, episode=None, location=None, hosp
 			INSERT INTO clin.procedure (
 				fk_encounter,
 				fk_episode,
+				soap_cat,
 				clin_where,
 				fk_hospital_stay,
 				narrative
 			) VALUES (
 				%(enc)s,
 				%(epi)s,
+				'p',
 				gm.nullify_empty_string(%(loc)s),
 				%(stay)s,
 				gm.nullify_empty_string(%(proc)s)
@@ -1565,6 +1970,7 @@ if __name__ == '__main__':
 		h_issue.close_expired_episode(ttl=7300)
 		h_issue = cHealthIssue(encounter = 1, name = u'post appendectomy/peritonitis')
 		print h_issue
+		print h_issue.format_as_journal()
 	#--------------------------------------------------------	
 	def test_episode():
 		print "\nepisode test"
@@ -1607,6 +2013,12 @@ if __name__ == '__main__':
 			print field, ':', encounter[field]
 		print "updatable:", encounter.get_updatable_fields()
 	#--------------------------------------------------------
+	def test_encounter2latex():
+		encounter = cEncounter(aPK_obj=1)
+		print encounter
+		print ""
+		print encounter.format_latex()
+	#--------------------------------------------------------
 	def test_performed_procedure():
 		procs = get_performed_procedures(patient = 12)
 		for proc in procs:
@@ -1637,6 +2049,7 @@ if __name__ == '__main__':
 	#test_health_issue()
 	#test_hospital_stay()
 	#test_performed_procedure()
-	test_diagnostic_certainty_classification_map()
+	#test_diagnostic_certainty_classification_map()
+	test_encounter2latex()
 #============================================================
 
