@@ -79,8 +79,8 @@ against. Please run GNUmed as a non-root user.
 	sys.exit(1)
 
 #----------------------------------------------------------
-current_client_version = u'0.8.6'
-current_client_branch = u'0.8'
+current_client_version = u'0.9.3'
+current_client_branch = u'0.9'
 
 _log = None
 _cfg = None
@@ -526,7 +526,7 @@ if ui_type == u'wxp':
 	else:
 		gmGuiMain.main()
 elif ui_type == u'web':
-	from Gnumed.ProxiedWeb import gmWebGuiServer
+	from Gnumed.proxiedpyjamas import gmWebGuiServer
 	gmWebGuiServer.main()
 
 elif ui_type == u'chweb':
