@@ -19,14 +19,17 @@ __license__ = 'GPL (details at http://www.gnu.org)'
 # stdlib
 import time, locale, sys, re as regex, os, codecs, types, datetime as pydt, logging
 
+
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmLoginInfo, gmExceptions, gmDateTime, gmBorg, gmI18N, gmLog2
+from Gnumed.pycommon.gmTools import prompted_input
 
 _log = logging.getLogger('gm.db')
 _log.info(__version__)
 
+
 # 3rd party
 try:
 	import psycopg2 as dbapi
@@ -100,7 +103,8 @@ known_schema_hashes = {
 	'v11': '03042ae24f3f92877d986fb0a6184d76',
 	'v12': '06183a6616db62257e22814007a8ed07',
 	'v13': 'fab7c1ae408a6530c47f9b5111a0841e',
-	'v14': 'e170d543f067d1ea60bfe9076b1560cf'
+	'v14': 'e170d543f067d1ea60bfe9076b1560cf',
+	'v15': '70012ff960b77ecdff4981c94b5b55b6'
 }
 
 map_schema_hash2version = {
@@ -116,7 +120,8 @@ map_schema_hash2version = {
 	'03042ae24f3f92877d986fb0a6184d76': 'v11',
 	'06183a6616db62257e22814007a8ed07': 'v12',
 	'fab7c1ae408a6530c47f9b5111a0841e': 'v13',
-	'e170d543f067d1ea60bfe9076b1560cf': 'v14'
+	'e170d543f067d1ea60bfe9076b1560cf': 'v14',
+	'70012ff960b77ecdff4981c94b5b55b6': 'v15'
 }
 
 map_client_branch2required_db_version = {
@@ -126,7 +131,8 @@ map_client_branch2required_db_version = {
 	u'0.5': u'v11',
 	u'0.6': u'v12',
 	u'0.7': u'v13',
-	u'0.8': u'v14'
+	u'0.8': u'v14',
+	u'0.9': u'v15'
 }
 
 # get columns and data types for a given table
@@ -301,12 +307,6 @@ def __detect_client_timezone(conn=None):
 # =======================================================================
 # login API
 # =======================================================================
-def __prompted_input(prompt, default=None):
-	usr_input = raw_input(prompt)
-	if usr_input == '':
-		return default
-	return usr_input
-#---------------------------------------------------
 def __request_login_params_tui():
 	"""Text mode request of database login parameters"""
 	import getpass
@@ -314,12 +314,12 @@ def __request_login_params_tui():
 
 	print "\nPlease enter the required login parameters:"
 	try:
-		login.host = __prompted_input("host ['' = non-TCP/IP]: ", '')
-		login.database = __prompted_input("database [gnumed_v14]: ", 'gnumed_v14')
-		login.user = __prompted_input("user name: ", '')
+		login.host = prompted_input(prompt = "host ('' = non-TCP/IP)", default = '')
+		login.database = prompted_input(prompt = "database", default = 'gnumed_v15')
+		login.user = prompted_input(prompt = "user name", default = '')
 		tmp = 'password for "%s" (not shown): ' % login.user
 		login.password = getpass.getpass(tmp)
-		login.port = __prompted_input("port [5432]: ", 5432)
+		login.port = prompted_input(prompt = "port", default = 5432)
 	except KeyboardInterrupt:
 		_log.warning("user cancelled text mode login dialog")
 		print "user cancelled text mode login dialog"
@@ -615,17 +615,135 @@ def get_col_names(link_obj=None, schema='public', table=None):
 	for row in rows:
 		cols.append(row[0])
 	return cols
+
+#------------------------------------------------------------------------
+# i18n functions
+#------------------------------------------------------------------------
+def export_translations_from_database(filename=None):
+	tx_file = codecs.open(filename, 'wb', 'utf8')
+	tx_file.write(u'-- GNUmed database string translations exported %s\n' % gmDateTime.pydt_now_here().strftime('%Y-%m-%d %H:%M'))
+	tx_file.write(u'-- - contains translations for each of [%s]\n' % u', '.join(get_translation_languages()))
+	tx_file.write(u'-- - user language is set to [%s]\n\n' % get_current_user_language())
+	tx_file.write(u'-- Please email this file to <gnumed-devel@gnu.org>.\n')
+	tx_file.write(u'-- ----------------------------------------------------------------------------------------------\n\n')
+	tx_file.write(u'set default_transaction_read_only to off\n\n')
+	tx_file.write(u'\\unset ON_ERROR_STOP\n\n')
+
+	cmd = u'SELECT lang, orig, trans FROM i18n.translations ORDER BY lang, orig'
+	rows, idx = run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = False)
+	for row in rows:
+		line = u"select i18n.upd_tx(quote_literal(E'%s'), quote_literal(E'%s'), quote_literal(E'%s'));\n" % (
+			row['lang'].replace("'", "\\'"),
+			row['orig'].replace("'", "\\'"),
+			row['trans'].replace("'", "\\'")
+		)
+		tx_file.write(line)
+	tx_file.write(u'\n')
+
+	tx_file.write(u'\set ON_ERROR_STOP 1\n')
+	tx_file.close()
+
+	return True
+#------------------------------------------------------------------------
+def delete_translation_from_database(link_obj=None, language=None, original=None):
+	cmd = u'DELETE FROM i18n.translations WHERE lang = %(lang)s AND orig = %(orig)s'
+	args = {'lang': language, 'orig': original}
+	run_rw_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}], return_data = False, end_tx = True)
+	return True
+
+#------------------------------------------------------------------------
+def update_translation_in_database(language=None, original=None, translation=None):
+	cmd = u'SELECT i18n.upd_tx(%(lang)s, %(orig)s, %(trans)s)'
+	args = {'lang': language, 'orig': original, 'trans': translation}
+	run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = False)
+	return args
+
 #------------------------------------------------------------------------
 def get_translation_languages():
 	rows, idx = run_ro_queries (
 		queries = [{'cmd': u'select distinct lang from i18n.translations'}]
 	)
 	return [ r[0] for r in rows ]
+
+#------------------------------------------------------------------------
+def get_database_translations(language=None, order_by=None):
+
+	args = {'lang': language}
+	_log.debug('language [%s]', language)
+
+	if order_by is None:
+		order_by = u'ORDER BY %s' % order_by
+	else:
+		order_by = u'ORDER BY lang, orig'
+
+	if language is None:
+		cmd = u"""
+		SELECT DISTINCT ON (orig, lang)
+			lang, orig, trans
+		FROM ((
+
+			-- strings stored as translation keys whether translated or not
+			SELECT
+				NULL as lang,
+				ik.orig,
+				NULL AS trans
+			FROM
+				i18n.keys ik
+
+		) UNION ALL (
+
+			-- already translated strings
+			SELECT
+				it.lang,
+				it.orig,
+				it.trans
+			FROM
+				i18n.translations it
+
+		)) as translatable_strings
+		%s""" % order_by
+	else:
+		cmd = u"""
+		SELECT DISTINCT ON (orig, lang)
+			lang, orig, trans
+		FROM ((
+
+			-- strings stored as translation keys whether translated or not
+			SELECT
+				%%(lang)s as lang,
+				ik.orig,
+				i18n._(ik.orig, %%(lang)s) AS trans
+			FROM
+				i18n.keys ik
+
+		) UNION ALL (
+
+			-- already translated strings
+			SELECT
+				%%(lang)s as lang,
+				it.orig,
+				i18n._(it.orig, %%(lang)s) AS trans
+			FROM
+				i18n.translations it
+
+		)) AS translatable_strings
+		%s""" % order_by
+
+	rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+
+	if rows is None:
+		_log.error('no translatable strings found')
+	else:
+		_log.debug('%s translatable strings found', len(rows))
+
+	return rows
+
 #------------------------------------------------------------------------
 def get_current_user_language():
 	cmd = u'select i18n.get_curr_lang()'
 	rows, idx = run_ro_queries(queries = [{'cmd': cmd}])
 	return rows[0][0]
+
 #------------------------------------------------------------------------
 def set_user_language(user=None, language=None):
 	"""Set the user language in the database.
@@ -860,6 +978,14 @@ def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=No
 	_log.debug('chunks to retrieve: [%s]' % needed_chunks)
 	_log.debug('remainder to retrieve: [%s] bytes' % remainder)
 
+	# try setting "bytea_output"
+	# - fails if not necessary
+	# - succeeds if necessary
+	try:
+		run_ro_queries(link_obj = conn, queries = [{'cmd': u"set bytea_output to 'escape'"}])
+	except dbapi.ProgrammingError:
+		_log.debug('failed to set bytea_output to "escacpe", not necessary')
+
 	# retrieve chunks, skipped if data size < chunk size,
 	# does this not carry the danger of cutting up multi-byte escape sequences ?
 	# no, since bytea is binary,
@@ -1140,6 +1266,49 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 	conn_close()
 
 	return (data, col_idx)
+#------------------------------------------------------------------------
+def run_insert(link_obj=None, schema=None, table=None, values=None, returning=None, end_tx=False, get_col_idx=False, verbose=False):
+	"""Generates SQL for an INSERT query.
+
+	values: dict of values keyed by field to insert them into
+	"""
+	if schema is None:
+		schema = u'public'
+
+	fields = values.keys()		# that way val_snippets and fields really should end up in the same order
+	val_snippets = []
+	for field in fields:
+		val_snippets.append(u'%%(%s)s' % field)
+
+	if returning is None:
+		returning = u''
+		return_data = False
+	else:
+		returning = u'\n\tRETURNING\n\t\t%s' % u', '.join(returning)
+		return_data = True
+
+	cmd = u"""\nINSERT INTO %s.%s (
+		%s
+	) VALUES (
+		%s
+	)%s""" % (
+		schema,
+		table,
+		u',\n\t\t'.join(fields),
+		u',\n\t\t'.join(val_snippets),
+		returning
+	)
+
+	_log.debug(u'running SQL: >>>%s<<<', cmd)
+
+	return run_rw_queries (
+		link_obj = link_obj,
+		queries = [{'cmd': cmd, 'args': values}],
+		end_tx = end_tx,
+		return_data = return_data,
+		get_col_idx = get_col_idx,
+		verbose = verbose
+	)
 # =======================================================================
 # connection handling API
 # -----------------------------------------------------------------------
@@ -1184,6 +1353,9 @@ def get_raw_connection(dsn=None, verbose=False, readonly=True):
 	if dsn is None:
 		dsn = get_default_dsn()
 
+	if u'host=salaam.homeunix' in dsn:
+		raise ValueError('The public database is not hosted by <salaam.homeunix.com> anymore.\n\nPlease point your configuration files to <publicdb.gnumed.de>.')
+
 	try:
 		conn = dbapi.connect(dsn=dsn, connection_factory=psycopg2.extras.DictConnection)
 	except dbapi.OperationalError, e:
@@ -1290,7 +1462,7 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 		_log.warning('for this to work properly the application MUST have called locale.setlocale() before')
 
 	# set connection properties
-	# 1) client encoding
+	# - client encoding
 	try:
 		conn.set_client_encoding(encoding)
 	except dbapi.OperationalError:
@@ -1299,7 +1471,7 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 			raise cEncodingError, (encoding, v), tb
 		raise
 
-	# 2) transaction isolation level
+	# - transaction isolation level
 	if readonly:
 		conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_READ_COMMITTED)
 		iso_level = u'read committed'
@@ -1311,26 +1483,45 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 
 	curs = conn.cursor()
 
-	# client time zone
+	# - client time zone
 	curs.execute(_sql_set_timezone, [_default_client_timezone])
 
-	# datestyle
+	# - datestyle
 	# regarding DMY/YMD handling: since we force *input* to
 	# ISO, too, the DMY/YMD setting is not needed
 	cmd = "set datestyle to 'ISO'"
 	curs.execute(cmd)
 
-	# SQL inheritance mode
+	# - SQL inheritance mode
 	cmd = 'set sql_inheritance to on'
 	curs.execute(cmd)
 
-	# version string
+	# - version string
 	global postgresql_version_string
 	if postgresql_version_string is None:
 		curs.execute('select version()')
 		postgresql_version_string = curs.fetchone()['version']
 		_log.info('PostgreSQL version (string): "%s"' % postgresql_version_string)
 
+	conn.commit()
+
+	# FIXME: remove this whole affair once either 9.0 is standard (Ubuntu 10 LTS is
+	# FIXME: PG 8.4, however!) or else when psycopg2 supports a workaround
+	#
+	# - bytea data format
+	# PG 9.0 switched to - by default - using "hex" rather than "escape",
+	# however, psycopg2's linked with a pre-9.0 libpq do assume "escape"
+	# as the transmission mode for bytea output,
+	# so try to set this setting back to "escape",
+	# if that's not possible the reason will be that PG < 9.0 does not support
+	# that setting - which also means we don't need it and can ignore the
+	# failure
+	cmd = "set bytea_output to 'escape'"
+	try:
+		curs.execute(cmd)
+	except dbapi.ProgrammingError:
+		_log.error('cannot set bytea_output format')
+
 	curs.close()
 	conn.commit()
 
@@ -1351,6 +1542,14 @@ def __noop():
 def _raise_exception_on_ro_conn_close():
 	raise TypeError(u'close() called on read-only connection')
 #-----------------------------------------------------------------------
+def log_database_access(action=None):
+	run_insert (
+		schema = u'gm',
+		table = u'access_log',
+		values = {u'user_action': action},
+		end_tx = True
+	)
+#-----------------------------------------------------------------------
 def sanity_check_time_skew(tolerance=60):
 	"""Check server time and local time to be within
 	the given tolerance of each other.
@@ -1889,6 +2088,15 @@ if __name__ == "__main__":
 		for line in get_schema_revision_history():
 			print u' - '.join(line)
 	#--------------------------------------------------------------------
+	def test_run_query():
+		gmDateTime.init()
+		args = {'dt': gmDateTime.pydt_max_here()}
+		cmd = u"select %(dt)s"
+
+		#cmd = u"select 'infinity'::timestamp with time zone"
+		rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		print rows
+	#--------------------------------------------------------------------
 	# run tests
 	#test_file2bytea()
 	#test_get_connection()
@@ -1904,6 +2112,7 @@ if __name__ == "__main__":
 	#test_keyword_expansion()
 	#test_get_foreign_key_details()
 	#test_set_user_language()
-	test_get_schema_revision_history()
+	#test_get_schema_revision_history()
+	test_run_query()
 
 # ======================================================================
