@@ -15,7 +15,7 @@ if __name__ == '__main__':
 from Gnumed.pycommon import gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmDateTime, gmTools, gmShellAPI, gmHooks
 from Gnumed.business import gmPerson, gmDocuments, gmEMRStructItems, gmSurgery
 from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin, gmEMRStructWidgets, gmListWidgets
-from Gnumed.wxGladeWidgets import wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl, wxgEditDocumentTypesPnl, wxgEditDocumentTypesDlg
+from Gnumed.wxGladeWidgets import wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl
 
 
 _log = logging.getLogger('gm.ui')
@@ -191,6 +191,19 @@ limit 25"""],
 
 		self.SetToolTipString(_('Enter a comment on the document.'))
 #============================================================
+# document type widgets
+#============================================================
+def manage_document_types(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	#dlg = gmDocumentWidgets.cEditDocumentTypesDlg(parent = self, id=-1)
+	dlg = cEditDocumentTypesDlg(parent = parent)
+	dlg.ShowModal()
+#============================================================
+from Gnumed.wxGladeWidgets import wxgEditDocumentTypesDlg
+
 class cEditDocumentTypesDlg(wxgEditDocumentTypesDlg.wxgEditDocumentTypesDlg):
 	"""A dialog showing a cEditDocumentTypesPnl."""
 
@@ -198,6 +211,8 @@ class cEditDocumentTypesDlg(wxgEditDocumentTypesDlg.wxgEditDocumentTypesDlg):
 		wxgEditDocumentTypesDlg.wxgEditDocumentTypesDlg.__init__(self, *args, **kwargs)
 
 #============================================================
+from Gnumed.wxGladeWidgets import wxgEditDocumentTypesPnl
+
 class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 	"""A panel grouping together fields to edit the list of document types."""
 
@@ -379,16 +394,16 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		del kwds['part']
 		wxgReviewDocPartDlg.wxgReviewDocPartDlg.__init__(self, *args, **kwds)
 
-		if isinstance(part, gmDocuments.cMedDocPart):
+		if isinstance(part, gmDocuments.cDocumentPart):
 			self.__part = part
 			self.__doc = self.__part.get_containing_document()
 			self.__reviewing_doc = False
-		elif isinstance(part, gmDocuments.cMedDoc):
+		elif isinstance(part, gmDocuments.cDocument):
 			self.__doc = part
 			self.__part = self.__doc.parts[0]
 			self.__reviewing_doc = True
 		else:
-			raise ValueError('<part> must be gmDocuments.cMedDoc or gmDocuments.cMedDocPart instance, got <%s>' % type(part))
+			raise ValueError('<part> must be gmDocuments.cDocument or gmDocuments.cDocumentPart instance, got <%s>' % type(part))
 
 		self.__init_ui_data()
 	#--------------------------------------------------------
@@ -1090,6 +1105,8 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	"""This wx.TreeCtrl derivative displays a tree view of stored medical documents.
 
 	It listens to document and patient changes and updated itself accordingly.
+
+	This acts on the current patient.
 	"""
 	_sort_modes = ['age', 'review', 'episode', 'type']
 	_root_node_labels = None
@@ -1125,7 +1142,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		node = self.GetSelection()
 		node_data = self.GetPyData(node)
 
-		if not isinstance(node_data, gmDocuments.cMedDocPart):
+		if not isinstance(node_data, gmDocuments.cDocumentPart):
 			return True
 
 		self.__display_part(part = node_data)
@@ -1251,6 +1268,9 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__doc_context_menu.Append(ID, _('Edit corresponding encounter'))
 		wx.EVT_MENU(self.__doc_context_menu, ID, self.__edit_encounter_details)
 
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Select corresponding encounter'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__select_encounter)
 
 #		self.__doc_context_menu.AppendSeparator()
 
@@ -1402,7 +1422,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		data2 = self.GetPyData(node2)
 
 		# doc node
-		if isinstance(data1, gmDocuments.cMedDoc):
+		if isinstance(data1, gmDocuments.cDocument):
 
 			date_field = 'clin_when'
 			#date_field = 'modified_when'
@@ -1462,7 +1482,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 				return 1
 
 		# part node
-		if isinstance(data1, gmDocuments.cMedDocPart):
+		if isinstance(data1, gmDocuments.cDocumentPart):
 			# compare sequence IDs (= "page" numbers)
 			# FIXME: wrong order ?
 			if data1['seq_idx'] < data2['seq_idx']:
@@ -1517,7 +1537,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			return None
 
 		# expand/collapse documents on activation
-		if isinstance(node_data, gmDocuments.cMedDoc):
+		if isinstance(node_data, gmDocuments.cDocument):
 			self.Toggle(node)
 			return True
 
@@ -1539,11 +1559,11 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			return None
 
 		# documents
-		if isinstance(self.__curr_node_data, gmDocuments.cMedDoc):
+		if isinstance(self.__curr_node_data, gmDocuments.cDocument):
 			self.__handle_doc_context()
 
 		# parts
-		if isinstance(self.__curr_node_data, gmDocuments.cMedDocPart):
+		if isinstance(self.__curr_node_data, gmDocuments.cDocumentPart):
 			self.__handle_part_context()
 
 		del self.__curr_node_data
@@ -1618,16 +1638,16 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		cfg = gmCfg.cCfgSQL()
 
-		# get export directory for temporary files
-		tmp_dir = gmTools.coalesce (
-			cfg.get2 (
-				option = "horstspace.tmp_dir",
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
-				bias = 'workplace'
-			),
-			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
-		)
-		_log.debug("temporary directory [%s]", tmp_dir)
+#		# get export directory for temporary files
+#		tmp_dir = gmTools.coalesce (
+#			cfg.get2 (
+#				option = "horstspace.tmp_dir",
+#				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+#				bias = 'workplace'
+#			),
+#			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+#		)
+#		_log.debug("temporary directory [%s]", tmp_dir)
 
 		# determine database export chunk size
 		chunksize = int(
@@ -1648,7 +1668,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		# display it
 		successful, msg = part.display_via_mime (
-			tmpdir = tmp_dir,
+#			tmpdir = tmp_dir,
 			chunksize = chunksize,
 			block = block_during_view
 		)
@@ -1716,16 +1736,16 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		cfg = gmCfg.cCfgSQL()
 
-		# get export directory for temporary files
-		tmp_dir = gmTools.coalesce (
-			cfg.get2 (
-				option = "horstspace.tmp_dir",
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
-				bias = 'workplace'
-			),
-			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
-		)
-		_log.debug("temporary directory [%s]", tmp_dir)
+#		# get export directory for temporary files
+#		tmp_dir = gmTools.coalesce (
+#			cfg.get2 (
+#				option = "horstspace.tmp_dir",
+#				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+#				bias = 'workplace'
+#			),
+#			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+#		)
+#		_log.debug("temporary directory [%s]", tmp_dir)
 
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
@@ -1736,7 +1756,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		))
 
 		part_file = self.__curr_node_data.export_to_file (
-			aTempDir = tmp_dir,
+#			aTempDir = tmp_dir,
 			aChunkSize = chunksize
 		)
 
@@ -1774,10 +1794,19 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	# document level context menu handlers
 	#--------------------------------------------------------
+	def __select_encounter(self, evt):
+		enc = gmEMRStructWidgets.select_encounters (
+			parent = self,
+			patient = gmPerson.gmCurrentPatient()
+		)
+		if enc is None:
+			return
+		self.__curr_node_data['pk_encounter'] = enc['pk_encounter']
+		self.__curr_node_data.save()
+	#--------------------------------------------------------
 	def __edit_encounter_details(self, evt):
-		enc = gmEMRStructItems.cEncounter(aPK_obj=self.__curr_node_data['pk_encounter'])
-		dlg = gmEMRStructWidgets.cEncounterEditAreaDlg(parent=self, encounter=enc)
-		dlg.ShowModal()
+		enc = gmEMRStructItems.cEncounter(aPK_obj = self.__curr_node_data['pk_encounter'])
+		gmEMRStructWidgets.edit_encounter(parent = self, encounter = enc)
 	#--------------------------------------------------------
 	def __process_doc(self, action=None, l10n_action=None):
 
@@ -1805,16 +1834,16 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		cfg = gmCfg.cCfgSQL()
 
-		# get export directory for temporary files
-		tmp_dir = gmTools.coalesce (
-			cfg.get2 (
-				option = "horstspace.tmp_dir",
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
-				bias = 'workplace'
-			),
-			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
-		)
-		_log.debug("temporary directory [%s]", tmp_dir)
+#		# get export directory for temporary files
+#		tmp_dir = gmTools.coalesce (
+#			cfg.get2 (
+#				option = "horstspace.tmp_dir",
+#				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+#				bias = 'workplace'
+#			),
+#			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+#		)
+#		_log.debug("temporary directory [%s]", tmp_dir)
 
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
@@ -1825,7 +1854,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		))
 
 		part_files = self.__curr_node_data.export_parts_to_files (
-			export_dir = tmp_dir,
+#			export_dir = tmp_dir,
 			chunksize = chunksize
 		)
 
