@@ -24,13 +24,10 @@ import wx.lib.pubsub as wxps
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmI18N, gmMatchProvider, gmDispatcher, gmTools, gmDateTime, gmCfg, gmExceptions
-from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmSurgery
+from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmSurgery, gmPersonSearch
 from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmListWidgets, gmEditArea, gmPatSearchWidgets
 from Gnumed.wxGladeWidgets import wxgIssueSelectionDlg, wxgMoveNarrativeDlg
-from Gnumed.wxGladeWidgets import wxgHealthIssueEditAreaPnl
-from Gnumed.wxGladeWidgets import wxgEncounterEditAreaPnl, wxgEncounterEditAreaDlg
 from Gnumed.wxGladeWidgets import wxgEncounterTypeEditAreaPnl
-from Gnumed.wxGladeWidgets import wxgEpisodeEditAreaPnl
 
 
 _log = logging.getLogger('gm.ui')
@@ -65,7 +62,19 @@ def manage_performed_procedures(parent=None):
 
 		items = [
 			[
-				p['clin_when'].strftime('%Y-%m-%d'),
+				u'%s%s' % (
+					p['clin_when'].strftime('%Y-%m-%d'),
+					gmTools.bool2subst (
+						p['is_ongoing'],
+						_(' (ongoing)'),
+						gmTools.coalesce (
+							initial = p['clin_end'],
+							instead = u'',
+							template_initial = u' - %s',
+							function_initial = ('strftime', u'%Y-%m-%d')
+						)
+					)
+				),
 				p['clin_where'],
 				p['episode'],
 				p['performed_procedure']
@@ -115,17 +124,36 @@ class cProcedureEAPnl(wxgProcedureEAPnl.wxgProcedureEAPnl, gmEditArea.cGenericEd
 		self._PRW_hospital_stay.add_callback_on_lose_focus(callback = self._on_hospital_stay_lost_focus)
 		self._PRW_hospital_stay.set_context(context = 'pat', val = gmPerson.gmCurrentPatient().ID)
 		self._PRW_location.add_callback_on_lose_focus(callback = self._on_location_lost_focus)
+		self._DPRW_date.add_callback_on_lose_focus(callback = self._on_start_lost_focus)
+		self._DPRW_end.add_callback_on_lose_focus(callback = self._on_end_lost_focus)
 
 		# location
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			queries = [
 u"""
-select distinct on (clin_where) clin_where, clin_where
-from clin.procedure
-where clin_where %(fragment_condition)s
-order by clin_where
-limit 25
-"""			]
+SELECT DISTINCT ON (data) data, location
+FROM (
+	SELECT
+		clin_where as data,
+		clin_where as location
+	FROM
+		clin.procedure
+	WHERE
+		clin_where %(fragment_condition)s
+
+		UNION ALL
+
+	SELECT
+		narrative as data,
+		narrative as location
+	FROM
+		clin.hospital_stay
+	WHERE
+		narrative %(fragment_condition)s
+) as union_result
+ORDER BY data
+LIMIT 25"""
+			]
 		)
 		mp.setThresholds(2, 4, 6)
 		self._PRW_location.matcher = mp
@@ -145,15 +173,18 @@ limit 25
 		self._PRW_procedure.matcher = mp
 	#----------------------------------------------------------------
 	def _on_hospital_stay_lost_focus(self):
-		if self._PRW_hospital_stay.GetData() is None:
+		stay = self._PRW_hospital_stay.GetData()
+		if stay is None:
 			self._PRW_hospital_stay.SetText()
 			self._PRW_location.Enable(True)
 			self._PRW_episode.Enable(True)
+			self._LBL_hospital_details.SetLabel(u'')
 		else:
 			self._PRW_location.SetText()
 			self._PRW_location.Enable(False)
 			self._PRW_episode.SetText()
 			self._PRW_episode.Enable(False)
+			self._LBL_hospital_details.SetLabel(gmEMRStructItems.cHospitalStay(aPK_obj = stay).format())
 	#----------------------------------------------------------------
 	def _on_location_lost_focus(self):
 		if self._PRW_location.GetValue().strip() == u'':
@@ -162,8 +193,44 @@ limit 25
 		else:
 			self._PRW_hospital_stay.SetText()
 			self._PRW_hospital_stay.Enable(False)
+			self._PRW_hospital_stay.display_as_valid(True)
 #			self._PRW_episode.Enable(True)
 	#----------------------------------------------------------------
+	def _on_start_lost_focus(self):
+		if not self._DPRW_date.is_valid_timestamp():
+			return
+		end = self._DPRW_end.GetData()
+		if end is None:
+			return
+		end = end.get_pydt()
+		start = self._DPRW_date.GetData().get_pydt()
+		if start < end:
+			return
+		self._DPRW_date.display_as_valid(False)
+	#----------------------------------------------------------------
+	def _on_end_lost_focus(self):
+		end = self._DPRW_end.GetData()
+		if end is None:
+			self._CHBOX_ongoing.Enable(True)
+			self._DPRW_end.display_as_valid(True)
+		else:
+			self._CHBOX_ongoing.Enable(False)
+			end = end.get_pydt()
+			now = gmDateTime.pydt_now_here()
+			if end > now:
+				self._CHBOX_ongoing.SetValue(True)
+			else:
+				self._CHBOX_ongoing.SetValue(False)
+			start = self._DPRW_date.GetData()
+			if start is None:
+				self._DPRW_end.display_as_valid(True)
+			else:
+				start = start.get_pydt()
+				if end > start:
+					self._DPRW_end.display_as_valid(True)
+				else:
+					self._DPRW_end.display_as_valid(False)
+	#----------------------------------------------------------------
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
 	def _valid_for_save(self):
@@ -176,6 +243,22 @@ limit 25
 		else:
 			self._DPRW_date.display_as_valid(True)
 
+		end = self._DPRW_end.GetData()
+		self._DPRW_end.display_as_valid(True)
+		if end is not None:
+			end = end.get_pydt()
+			start = self._DPRW_end.GetData()
+			if start is not None:
+				start = start.get_pydt()
+				if end < start:
+					has_errors = True
+					self._DPRW_end.display_as_valid(False)
+			if self._CHBOX_ongoing.IsChecked():
+				now = gmDateTime.pydt_now_here()
+				if end < now:
+					has_errors = True
+					self._DPRW_end.display_as_valid(False)
+
 		if self._PRW_hospital_stay.GetData() is None:
 			if self._PRW_episode.GetData() is None:
 				self._PRW_episode.display_as_valid(False)
@@ -231,7 +314,13 @@ limit 25
 			hospital_stay = stay,
 			procedure = self._PRW_procedure.GetValue().strip()
 		)
+
 		proc['clin_when'] = self._DPRW_date.data.get_pydt()
+		if self._DPRW_end.GetData() is None:
+			proc['clin_end'] = None
+		else:
+			proc['clin_end'] = self._DPRW_end.GetData().get_pydt()
+		proc['is_ongoing'] = self._CHBOX_ongoing.IsChecked()
 		proc.save()
 
 		self.data = proc
@@ -241,6 +330,13 @@ limit 25
 	def _save_as_update(self):
 		self.data['clin_when'] = self._DPRW_date.data.get_pydt()
 
+		if self._DPRW_end.GetData() is None:
+			self.data['clin_end'] = None
+		else:
+			self.data['clin_end'] = self._DPRW_end.GetData().get_pydt()
+
+		self.data['is_ongoing'] = self._CHBOX_ongoing.IsChecked()
+
 		if self._PRW_hospital_stay.GetData() is None:
 			self.data['pk_hospital_stay'] = None
 			self.data['clin_where'] = self._PRW_location.GetValue().strip()
@@ -258,26 +354,43 @@ limit 25
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
 		self._DPRW_date.SetText()
+		self._DPRW_end.SetText()
+		self._CHBOX_ongoing.SetValue(False)
+		self._CHBOX_ongoing.Enable(True)
 		self._PRW_hospital_stay.SetText()
 		self._PRW_location.SetText()
 		self._PRW_episode.SetText()
 		self._PRW_procedure.SetText()
 
-		self._DPRW_date.SetFocus()
+		self._PRW_procedure.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		self._DPRW_date.SetData(data = self.data['clin_when'])
+		if self.data['clin_end'] is None:
+			self._DPRW_end.SetText()
+			self._CHBOX_ongoing.Enable(True)
+			self._CHBOX_ongoing.SetValue(self.data['is_ongoing'])
+		else:
+			self._DPRW_end.SetData(data = self.data['clin_end'])
+			self._CHBOX_ongoing.Enable(False)
+			now = gmDateTime.pydt_now_here()
+			if self.data['clin_end'] > now:
+				self._CHBOX_ongoing.SetValue(True)
+			else:
+				self._CHBOX_ongoing.SetValue(False)
 		self._PRW_episode.SetText(value = self.data['episode'], data = self.data['pk_episode'])
 		self._PRW_procedure.SetText(value = self.data['performed_procedure'], data = self.data['performed_procedure'])
 
 		if self.data['pk_hospital_stay'] is None:
 			self._PRW_hospital_stay.SetText()
+			self._LBL_hospital_details.SetLabel(u'')
 			self._PRW_location.SetText(value = self.data['clin_where'], data = self.data['clin_where'])
 		else:
 			self._PRW_hospital_stay.SetText(value = self.data['clin_where'], data = self.data['pk_hospital_stay'])
+			self._LBL_hospital_details.SetLabel(gmEMRStructItems.cHospitalStay(aPK_obj = self.data['pk_hospital_stay']).format())
 			self._PRW_location.SetText()
 
-		self._DPRW_date.SetFocus()
+		self._PRW_procedure.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		self._refresh_as_new()
@@ -289,11 +402,30 @@ limit 25
 			self._PRW_hospital_stay.SetText(value = self.data['clin_where'], data = self.data['pk_hospital_stay'])
 			self._PRW_location.SetText()
 
-		self._DPRW_date.SetFocus()
+		self._PRW_procedure.SetFocus()
+	#----------------------------------------------------------------
+	# event handlers
 	#----------------------------------------------------------------
 	def _on_add_hospital_stay_button_pressed(self, evt):
+		# FIXME: this would benefit from setting the created stay
 		edit_hospital_stay(parent = self.GetParent())
 		evt.Skip()
+	#----------------------------------------------------------------
+	def _on_ongoing_checkbox_checked(self, event):
+		if self._CHBOX_ongoing.IsChecked():
+			end = self._DPRW_end.GetData()
+			if end is None:
+				self._DPRW_end.display_as_valid(True)
+			else:
+				end = end.get_pydt()
+				now = gmDateTime.pydt_now_here()
+				if end > now:
+					self._DPRW_end.display_as_valid(True)
+				else:
+					self._DPRW_end.display_as_valid(False)
+		else:
+			self._DPRW_end.is_valid_timestamp()
+		event.Skip()
 #================================================================
 # hospital stay related widgets/functions
 #----------------------------------------------------------------
@@ -323,7 +455,7 @@ def manage_hospital_stays(parent=None):
 		items = [
 			[
 				s['admission'].strftime('%Y-%m-%d'),
-				gmTools.coalesce(s['discharge'], u''),
+				gmTools.coalesce(s['discharge'], u'', function_initial = ('strftime', '%Y-%m-%d')),
 				s['episode'],
 				gmTools.coalesce(s['hospital'], u'')
 			] for s in stays
@@ -418,48 +550,44 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
 	def _valid_for_save(self):
-		if not self._DP_admission.GetValue().IsValid():
-			self._DP_admission.SetBackgroundColour(gmPhraseWheel.color_prw_invalid)
+
+		valid = True
+
+		if not self._PRW_admission.is_valid_timestamp(allow_empty = False):
+			valid = False
 			wxps.Publisher().sendMessage (
 				topic = 'statustext',
 				data = {'msg': _('Missing admission data. Cannot save hospital stay.'), 'beep': True}
 			)
-			return False
-		else:
-			self._DP_admission.SetBackgroundColour(gmPhraseWheel.color_prw_valid)
 
-		if self._DP_discharge.GetValue().IsValid():
-			if not self._DP_discharge.GetValue().IsLaterThan(self._DP_admission.GetValue()):
-				self._DP_discharge.SetBackgroundColour(gmPhraseWheel.color_prw_invalid)
-				wxps.Publisher().sendMessage (
-					topic = 'statustext',
-					data = {'msg': _('Discharge date must be empty or later than admission. Cannot save hospital stay.'), 'beep': True}
-				)
-				return False
+		if self._PRW_discharge.is_valid_timestamp(allow_empty = True):
+			if self._PRW_discharge.date is not None:
+				if not self._PRW_discharge.date > self._PRW_admission.date:
+					valid = False
+					self._PRW_discharge.display_as_valid(False)
+					wxps.Publisher().sendMessage (
+						topic = 'statustext',
+						data = {'msg': _('Discharge date must be empty or later than admission. Cannot save hospital stay.'), 'beep': True}
+					)
 
 		if self._PRW_episode.GetValue().strip() == u'':
+			valid = False
 			self._PRW_episode.display_as_valid(False)
 			wxps.Publisher().sendMessage (
 				topic = 'statustext',
 				data = {'msg': _('Must select an episode or enter a name for a new one. Cannot save hospital stay.'), 'beep': True}
 			)
-			return False
 
-		return True
+		return (valid is True)
 	#----------------------------------------------------------------
 	def _save_as_new(self):
 
 		pat = gmPerson.gmCurrentPatient()
 		emr = pat.get_emr()
-
-		stay = gmEMRStructItems.create_hospital_stay (
-			encounter = emr.active_encounter['pk_encounter'],
-			episode = self._PRW_episode.GetData(can_create = True)
-		)
+		stay = emr.add_hospital_stay(episode = self._PRW_episode.GetData(can_create = True))
 		stay['hospital'] = gmTools.none_if(self._PRW_hospital.GetValue().strip(), u'')
-		stay['admission'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_admission.GetValue())
-		if self._DP_discharge.GetValue().IsValid():
-			stay['discharge'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_discharge.GetValue())
+		stay['admission'] = self._PRW_admission.GetData()
+		stay['discharge'] = self._PRW_discharge.GetData()
 		stay.save_payload()
 
 		self.data = stay
@@ -469,9 +597,8 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 
 		self.data['pk_episode'] = self._PRW_episode.GetData(can_create = True)
 		self.data['hospital'] = gmTools.none_if(self._PRW_hospital.GetValue().strip(), u'')
-		self.data['admission'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_admission.GetValue())
-		if self._DP_discharge.GetValue().IsValid():
-			self.data['discharge'] = gmDateTime.wxDate2py_dt(wxDate = self._DP_discharge.GetValue())
+		self.data['admission'] = self._PRW_admission.GetData()
+		self.data['discharge'] = self._PRW_discharge.GetData()
 		self.data.save_payload()
 
 		return True
@@ -479,8 +606,8 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 	def _refresh_as_new(self):
 		self._PRW_hospital.SetText(value = u'')
 		self._PRW_episode.SetText(value = u'')
-		self._DP_admission.SetValue(dt = wx.DateTime.UNow())
-		#self._DP_discharge.SetValue(dt = None)
+		self._PRW_admission.SetText(data = pydt.datetime.now())
+		self._PRW_discharge.SetText()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		if self.data['hospital'] is not None:
@@ -489,10 +616,8 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 		if self.data['pk_episode'] is not None:
 			self._PRW_episode.SetText(value = self.data['episode'], data = self.data['pk_episode'])
 
-		self._DP_admission.SetValue(gmDateTime.py_dt2wxDate(py_dt = self.data['admission'], wx = wx))
-
-		if self.data['discharge'] is not None:
-			self._DP_discharge.SetValue(gmDateTime.py_dt2wxDate(py_dt = self.data['discharge'], wx = wx))
+		self._PRW_admission.SetText(data = self.data['admission'])
+		self._PRW_discharge.SetText(data = self.data['discharge'])
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		print "this was not expected to be used in this edit area"
@@ -508,6 +633,8 @@ def start_new_encounter(emr=None):
 		_('Start of new encounter')
 	)
 #----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgEncounterEditAreaDlg
+
 def edit_encounter(parent=None, encounter=None):
 	if parent is None:
 		parent = wx.GetApp().GetTopWindow()
@@ -520,7 +647,7 @@ def edit_encounter(parent=None, encounter=None):
 	dlg.Destroy()
 	return False
 #----------------------------------------------------------------
-def select_encounters(parent=None, patient=None, single_selection=True, encounters=None):
+def select_encounters(parent=None, patient=None, single_selection=True, encounters=None, ignore_OK_button=False):
 
 	if patient is None:
 		patient = gmPerson.gmCurrentPatient()
@@ -536,14 +663,14 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 
 	#--------------------
 	def refresh(lctrl):
-		if encounters is not None:
-			encs = encounters
-		else:
+		if encounters is None:
 			encs = emr.get_encounters()
+		else:
+			encs = encounters
 
 		items = [
 			[
-				e['started'].strftime('%x %H:%M'),
+				e['started'].strftime('%x  %H:%M'),
 				e['last_affirmed'].strftime('%H:%M'),
 				e['l10n_type'],
 				gmTools.coalesce(e['reason_for_encounter'], u''),
@@ -556,7 +683,11 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 		lctrl.set_string_items(items = items)
 		lctrl.set_data(data = encs)
 	#--------------------
-	def edit(enc = None):
+	def new():
+		enc = gmEMRStructItems.create_encounter(fk_patient = patient.ID)
+		return edit_encounter(parent = parent, encounter = enc)
+	#--------------------
+	def edit(enc=None):
 		return edit_encounter(parent = parent, encounter = enc)
 	#--------------------
 	return gmListWidgets.get_choices_from_list (
@@ -567,7 +698,9 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 		can_return_empty = True,
 		single_selection = single_selection,
 		refresh_callback = refresh,
-		edit_callback = edit
+		edit_callback = edit,
+		new_callback = new,
+		ignore_OK_button = ignore_OK_button
 	)
 #----------------------------------------------------------------
 def ask_for_encounter_continuation(msg=None, caption=None, encounter=None, parent=None):
@@ -773,6 +906,8 @@ class cEncounterTypeEditAreaPnl(wxgEncounterTypeEditAreaPnl.wxgEncounterTypeEdit
 #	def __register_interests(self):
 #		return
 #----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgEncounterEditAreaPnl
+
 class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 
 	def __init__(self, *args, **kwargs):
@@ -1282,6 +1417,8 @@ limit 30"""
 			self.set_context('pat', patient.ID)
 		return True
 #----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgEpisodeEditAreaPnl
+
 class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPnl.wxgEpisodeEditAreaPnl):
 
 	def __init__(self, *args, **kwargs):
@@ -1319,6 +1456,7 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 		emr = pat.get_emr()
 
 		epi = emr.add_episode(episode_name = self._PRW_description.GetValue().strip())
+		epi['summary'] = self._TCTRL_summary.GetValue().strip()
 		epi['episode_open'] = not self._CHBOX_closed.IsChecked()
 		epi['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
 
@@ -1346,6 +1484,7 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 	def _save_as_update(self):
 
 		self.data['description'] = self._PRW_description.GetValue().strip()
+		self.data['summary'] = self._TCTRL_summary.GetValue().strip()
 		self.data['episode_open'] = not self._CHBOX_closed.IsChecked()
 		self.data['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
 
@@ -1375,6 +1514,7 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 		self._TCTRL_patient.SetValue(ident.get_description_gender())
 		self._PRW_issue.SetText()
 		self._PRW_description.SetText()
+		self._TCTRL_summary.SetValue(u'')
 		self._PRW_classification.SetText()
 		self._CHBOX_closed.SetValue(False)
 	#----------------------------------------------------------------
@@ -1387,6 +1527,8 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 
 		self._PRW_description.SetText(self.data['description'], data=self.data['description'])
 
+		self._TCTRL_summary.SetValue(gmTools.coalesce(self.data['summary'], u''))
+
 		if self.data['diagnostic_certainty_classification'] is not None:
 			self._PRW_classification.SetData(data = self.data['diagnostic_certainty_classification'])
 
@@ -1569,6 +1711,8 @@ class cIssueSelectionDlg(wxgIssueSelectionDlg.wxgIssueSelectionDlg):
 			return False
 		return True
 #------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgHealthIssueEditAreaPnl
+
 class cHealthIssueEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPnl):
 	"""Panel encapsulating health issue edit area functionality."""
 
@@ -1634,6 +1778,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		self._PRW_age_noted.add_callback_on_modified(self._on_modified_age_noted)
 		self._PRW_year_noted.add_callback_on_modified(self._on_modified_year_noted)
 
+		self._PRW_year_noted.Enable(True)
+
 		self.data = issue
 	#----------------------------------------------------------------
 	# generic Edit Area mixin API
@@ -1671,6 +1817,7 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			side += u'd'
 		issue['laterality'] = side
 
+		issue['summary'] = self._TCTRL_summary.GetValue().strip()
 		issue['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
 		issue['grouping'] = self._PRW_grouping.GetValue().strip()
 		issue['is_active'] = self._ChBOX_active.GetValue()
@@ -1684,17 +1831,10 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		issue.save()
 
-		narr = self._TCTRL_notes.GetValue().strip()
-		if narr != u'':
-			epi = emr.add_episode(episode_name = _('inception notes'), pk_health_issue = issue['pk_health_issue'])
-			emr.add_clin_narrative(note = narr, soap_cat = 's', episode = epi)
-
 		self.data = issue
-
 		return True
 	#----------------------------------------------------------------
 	def _save_as_update(self):
-		# update self.data and save the changes
 
 		self.data['description'] = self._PRW_condition.GetValue().strip()
 
@@ -1705,6 +1845,7 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			side += u'd'
 		self.data['laterality'] = side
 
+		self.data['summary'] = self._TCTRL_summary.GetValue().strip()
 		self.data['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
 		self.data['grouping'] = self._PRW_grouping.GetValue().strip()
 		self.data['is_active'] = bool(self._ChBOX_active.GetValue())
@@ -1718,13 +1859,6 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		self.data.save()
 
-		narr = self._TCTRL_notes.GetValue().strip()
-		if narr != '':
-			pat = gmPerson.gmCurrentPatient()
-			emr = pat.get_emr()
-			epi = emr.add_episode(episode_name = _('inception notes'), pk_health_issue = self.data['pk_health_issue'])
-			emr.add_clin_narrative(note = narr, soap_cat = 's', episode = epi)
-
 		# FIXME: handle is_operation
 		return True
 	#----------------------------------------------------------------
@@ -1734,7 +1868,7 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		self._ChBOX_right.SetValue(0)
 		self._PRW_classification.SetText()
 		self._PRW_grouping.SetText()
-		self._TCTRL_notes.SetValue(u'')
+		self._TCTRL_summary.SetValue(u'')
 		self._PRW_age_noted.SetText()
 		self._PRW_year_noted.SetText()
 		self._ChBOX_active.SetValue(0)
@@ -1760,7 +1894,7 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		self._PRW_classification.SetData(data = self.data['diagnostic_certainty_classification'])
 		self._PRW_grouping.SetText(gmTools.coalesce(self.data['grouping'], u''))
-		self._TCTRL_notes.SetValue('')
+		self._TCTRL_summary.SetValue(gmTools.coalesce(self.data['summary'], u''))
 
 		if self.data['age_noted'] is None:
 			self._PRW_age_noted.SetText()
@@ -2029,7 +2163,7 @@ if __name__ == '__main__':
 		gmDateTime.init()
 
 		# obtain patient
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		if pat is None:
 			print "No patient. Exiting gracefully..."
 			sys.exit(0)
