@@ -14,11 +14,25 @@ import wx
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmI18N, gmGuiBroker, gmExceptions, gmBorg, gmTools
-from Gnumed.pycommon import gmCfg2, gmDateTime
-from Gnumed.business import gmPerson, gmDemographicRecord, gmMedication, gmPathLab
-from Gnumed.business import gmVaccination
-from Gnumed.wxpython import gmGuiHelpers, gmPlugin, gmPatSearchWidgets, gmNarrativeWidgets
+from Gnumed.pycommon import gmI18N
+if __name__ == '__main__':
+	gmI18N.activate_locale()
+	gmI18N.install_domain()
+from Gnumed.pycommon import gmGuiBroker
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmBorg
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmCfg2
+from Gnumed.pycommon import gmDateTime
+
+from Gnumed.business import gmPerson, gmDemographicRecord, gmMedication, gmPathLab, gmPersonSearch
+from Gnumed.business import gmVaccination, gmPersonSearch
+
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmNarrativeWidgets
+from Gnumed.wxpython import gmPatSearchWidgets
+from Gnumed.wxpython import gmPlugin
+from Gnumed.wxpython import gmEMRStructWidgets
 
 
 _log = logging.getLogger('gm.scripting')
@@ -45,7 +59,17 @@ known_placeholders = [
 # those must satisfy the pattern "$name::args::optional length$" when used
 known_variant_placeholders = [
 	u'soap',
-	u'progress_notes',
+	u'progress_notes',			# "data" holds: categories//template
+								# 	categories: string with "soap ", " " == None == admin
+								#	template:	u'something %s something'		(do not include // in template !)
+	u'emr_journal',				# "data" format:   <categories>//<template>//<line length>//<time range>//<target format>
+								#	categories:	   string with any of "s", "o", "a", "p", " ";
+								#				   (" " == None == admin category)
+								#	template:	   something %s something else
+								#				   (Do not include // in the template !)
+								#	line length:   the length of individual lines, not the total placeholder length
+								#	time range:	   the number of weeks going back in time
+								#	target format: "tex" or anything else, if "tex", data will be tex-escaped
 	u'date_of_birth',
 	u'adr_street',				# "data" holds: type of address
 	u'adr_number',
@@ -62,7 +86,9 @@ known_variant_placeholders = [
 	u'allergies',				# "data" holds: line template, one allergy per line
 	u'allergy_list',			# "data" holds: template per allergy, allergies on one line
 	u'problems',				# "data" holds: line template, one problem per line
-	u'name'						# "data" holds: template for name parts arrangement
+	u'name',					# "data" holds: template for name parts arrangement
+	u'free_text',				# show a dialog for entering some free text
+	u'soap_for_encounters'		# "data" holds: soap cats // strftime date format
 ]
 
 default_placeholder_regex = r'\$<.+?>\$'				# this one works (except that OOo cannot be non-greedy |-( )
@@ -151,18 +177,18 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 				return None
 
 		# variable placeholders
-		parts = placeholder.split('::', 2)
+		parts = placeholder.split('::')
 		if len(parts) == 2:
 			name, data = parts
 			lng = None
-		elif len(parts) == 3:
+		if len(parts) == 3:
 			name, data, lng = parts
 			try:
 				lng = int(lng)
 			except:
 				_log.exception('placeholder length definition error: %s, discarding length', original_placeholder)
 				lng = None
-		else:
+		if len(parts) > 3:
 			_log.warning('invalid placeholder layout: %s', original_placeholder)
 			if self.debug:
 				return self.invalid_placeholder_template % original_placeholder
@@ -288,28 +314,155 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 	#--------------------------------------------------------
 	# variant handlers
 	#--------------------------------------------------------
-	def _get_variant_progress_notes(self, data=None):
-		return self._get_variant_soap(data=data)
-	#--------------------------------------------------------
-	def _get_variant_soap(self, data=None):
-		if data is None:
-			cats = list(data)
-			template = u'%s'
-		else:
-			parts = data.split('//', 2)
-			if len(parts) == 1:
-				cats = list(parts)
-				template = u'%s'
-			else:
-				cats = list(parts[0])
-				template = parts[1]
+	def _get_variant_soap_for_encounters(self, data=None):
+		"""Select encounters from list and format SOAP thereof.
 
-		narr = gmNarrativeWidgets.select_narrative_from_episodes(soap_cats = cats)
+		data: soap_cats (' ' -> None -> admin) // date format
+		"""
+		# defaults
+		cats = None
+		date_format = None
+
+		if data is not None:
+			data_parts = data.split('//')
+
+			# part[0]: categories
+			if len(data_parts[0]) > 0:
+				cats = []
+				if u' ' in data_parts[0]:
+					cats.append(None)
+					data_parts[0] = data_parts[0].replace(u' ', u'')
+				cats.extend(list(data_parts[0]))
+
+			# part[1]: date format
+			if len(data_parts) > 0:
+				if len(data_parts[1]) > 0:
+					date_format = data_parts[1]
+
+		encounters = gmEMRStructWidgets.select_encounters(single_selection = False)
+
+		chunks = []
+		for enc in encounters:
+			chunks.append(enc.format_latex(date_format = date_format, soap_cats = cats))
+
+		return u''.join(chunks)
+	#--------------------------------------------------------
+	def _get_variant_emr_journal(self, data=None):
+		# default: all categories, neutral template
+		cats = list(u'soap')
+		cats.append(None)
+		template = u'%s'
+		interactive = True
+		line_length = 9999
+		target_format = None
+		time_range = None
+
+		if data is not None:
+			data_parts = data.split('//')
+
+			# part[0]: categories
+			cats = []
+			# ' ' -> None == admin
+			for c in list(data_parts[0]):
+				if c == u' ':
+					c = None
+				cats.append(c)
+			# '' -> SOAP + None
+			if cats == u'':
+				cats = list(u'soap').append(None)
+
+			# part[1]: template
+			if len(data_parts) > 0:
+				template = data_parts[1]
+
+			# part[2]: line length
+			if len(data_parts) > 1:
+				try:
+					line_length = int(data_parts[2])
+				except:
+					line_length = 9999
+
+			# part[3]: weeks going back in time
+			if len(data_parts) > 2:
+				try:
+					time_range = 7 * int(data_parts[3])
+				except:
+					time_range = None
+
+			# part[4]: output format
+			if len(data_parts) > 3:
+				target_format = data_parts[4]
+
+		# FIXME: will need to be a generator later on
+		narr = self.pat.get_emr().get_as_journal(soap_cats = cats, time_range = time_range)
 
 		if len(narr) == 0:
 			return u''
 
-		narr = [ template % n['narrative'] for n in narr ]
+		if target_format == u'tex':
+			keys = narr[0].keys()
+			lines = []
+			line_dict = {}
+			for n in narr:
+				for key in keys:
+					if isinstance(n[key], basestring):
+						line_dict[key] = gmTools.tex_escape_string(text = n[key])
+						continue
+					line_dict[key] = n[key]
+				try:
+					lines.append((template % line_dict)[:line_length])
+				except KeyError:
+					return u'invalid key in template [%s], valid keys: %s]' % (template, str(keys))
+		else:
+			try:
+				lines = [ (template % n)[:line_length] for n in narr ]
+			except KeyError:
+				return u'invalid key in template [%s], valid keys: %s]' % (template, str(narr[0].keys()))
+
+		return u'\n'.join(lines)
+	#--------------------------------------------------------
+	def _get_variant_progress_notes(self, data=None):
+		return self._get_variant_soap(data=data)
+	#--------------------------------------------------------
+	def _get_variant_soap(self, data=None):
+
+		# default: all categories, neutral template
+		cats = list(u'soap')
+		cats.append(None)
+		template = u'%s'
+
+		if data is not None:
+			data_parts = data.split('//')
+
+			# part[0]: categories
+			cats = []
+			# ' ' -> None == admin
+			for cat in list(data_parts[0]):
+				if cat == u' ':
+					cat = None
+				cats.append(cat)
+			# '' -> SOAP + None
+			if cats == u'':
+				cats = list(u'soap')
+				cats.append(None)
+
+			# part[1]: template
+			if len(data_parts) > 0:
+				template = data_parts[1]
+
+		#narr = gmNarrativeWidgets.select_narrative_from_episodes_new(soap_cats = cats)
+		narr = gmNarrativeWidgets.select_narrative_from_episodes(soap_cats = cats)
+
+		if narr is None:
+			return u''
+
+		if len(narr) == 0:
+			return u''
+
+		try:
+			narr = [ template % n['narrative'] for n in narr ]
+		except KeyError:
+			return u'invalid key in template [%s], valid keys: %s]' % (template, str(narr[0].keys()))
 
 		return u'\n'.join(narr)
 	#--------------------------------------------------------
@@ -484,6 +637,38 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 	def _get_variant_tex_escape(self, data=None):
 		return gmTools.tex_escape_string(text = data)
 	#--------------------------------------------------------
+	def _get_variant_free_text(self, data=u'tex//'):
+		# <data>:
+		#	format:	tex (only, currently)
+		#	message: shown in input dialog, must not contain "//" or "::"
+
+		format, msg = data.split('//')
+
+		dlg = gmGuiHelpers.cMultilineTextEntryDlg (
+			None,
+			-1,
+			title = _('Replacing <free_text> placeholder'),
+			msg = _('Below you can enter free text.\n\n [%s]') % msg
+		)
+		dlg.enable_user_formatting = True
+		decision = dlg.ShowModal()
+
+		if decision != wx.ID_SAVE:
+			dlg.Destroy()
+			return _('Text input cancelled by user.')
+
+		text = dlg.value.strip()
+		if dlg.is_user_formatted:
+			dlg.Destroy()
+			return text
+
+		dlg.Destroy()
+
+		if format == u'tex':
+			return gmTools.tex_escape_string(text = text)
+
+		return text
+	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
 
@@ -630,7 +815,7 @@ class cMacroPrimitives:
 		if self.__pat.locked:
 			_log.error('patient is already locked')
 			return (0, _('already locked into a patient'))
-		searcher = gmPerson.cPatientSearcher_SQL()
+		searcher = gmPersonSearch.cPatientSearcher_SQL()
 		if type(search_params) == types.DictType:
 			idents = searcher.get_identities(search_dict=search_params)
 			print "must use dto, not search_dict"
@@ -740,7 +925,7 @@ if __name__ == '__main__':
 		for placeholder in ['a', 'b']:
 			print handler[placeholder]
 
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		if pat is None:
 			return
 
@@ -805,7 +990,7 @@ if __name__ == '__main__':
 			'$<latest_vaccs_table::latex>$'
 		]
 
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		if pat is None:
 			return
 
@@ -880,7 +1065,7 @@ if __name__ == '__main__':
 
 			'\\noindent Patient: $<lastname>$, $<firstname>$',
 			'$<allergies::%(descriptor)s & %(l10n_type)s & {\\footnotesize %(reaction)s} \tabularnewline \hline >$',
-			'$<current_meds::		\item[%(substance)s] {\\footnotesize (%(brand)s)} %(preparation)s %(strength)s: %(schedule)s >$'
+			'$<current_meds::		\item[%(substance)s] {\\footnotesize (%(brand)s)} %(preparation)s %(amount)s%(unit)s: %(schedule)s >$'
 		]
 
 		tests = [
@@ -907,11 +1092,30 @@ if __name__ == '__main__':
 				print ' => "%s"' % p
 			print " "
 	#--------------------------------------------------------
+	def test_placeholder():
+
+		#ph = u'emr_journal::soap //%(date)s  %(modified_by)s  %(soap_cat)s  %(narrative)s//30::'
+		#ph = u'free_text::latex//placeholder test::9999'
+		ph = u'soap_for_encounters:://::9999'
+
+		handler = gmPlaceholderHandler()
+		handler.debug = True
+
+		pat = gmPersonSearch.ask_for_patient()
+		if pat is None:
+			return
+
+		gmPatSearchWidgets.set_active_patient(patient = pat)
+
+		app = wx.PyWidgetTester(size = (200, 50))
+		print u'%s => %s' % (ph, handler[ph])
+	#--------------------------------------------------------
 
 	#test_placeholders()
-	test_new_variant_placeholders()
+	#test_new_variant_placeholders()
 	#test_scripting()
 	#test_placeholder_regex()
+	test_placeholder()
 
 #=====================================================================
 
