@@ -6,6 +6,7 @@ __license__ = "GPL"
 
 
 import sys, logging, datetime as pyDT, decimal, os, webbrowser, subprocess, codecs
+import os.path
 
 
 import wx, wx.grid, wx.lib.hyperlink
@@ -13,8 +14,9 @@ import wx, wx.grid, wx.lib.hyperlink
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.business import gmPerson, gmPathLab, gmSurgery, gmLOINC, gmForms
+from Gnumed.business import gmPerson, gmPathLab, gmSurgery, gmLOINC, gmForms, gmPersonSearch
 from Gnumed.pycommon import gmTools, gmDispatcher, gmMatchProvider, gmDateTime, gmI18N, gmCfg, gmShellAPI
+from Gnumed.pycommon import gmNetworkTools
 from Gnumed.wxpython import gmRegetMixin, gmPhraseWheel, gmEditArea, gmGuiHelpers, gmListWidgets
 from Gnumed.wxpython import gmAuthWidgets, gmPatSearchWidgets, gmFormWidgets
 from Gnumed.wxGladeWidgets import wxgMeasurementsPnl, wxgMeasurementsReviewDlg
@@ -34,20 +36,17 @@ def update_loinc_reference_data():
 	gmDispatcher.send(signal = 'statustext', msg = _('Updating LOINC data can take quite a while...'), beep = True)
 
 	# download
-	downloaded = gmShellAPI.run_command_in_shell(command = 'gm-download_loinc', blocking = True)
+	downloaded, loinc_dir = gmNetworkTools.download_data_pack(url = 'http://www.gnumed.de/downloads/data/loinc/loinctab.zip')
 	if not downloaded:
 		wx.EndBusyCursor()
 		gmGuiHelpers.gm_show_warning (
 			aTitle = _('Downloading LOINC'),
-			aMessage = _(
-				'Running <gm-download_loinc> to retrieve\n'
-				'the latest LOINC data failed.\n'
-			)
+			aMessage = _('Error downloading the latest LOINC data.\n')
 		)
 		return False
 
-	# split and import
-	data_fname, license_fname = gmLOINC.split_LOINCDBTXT(input_fname = '/tmp/LOINCDB.TXT')
+	# split master data file
+	data_fname, license_fname = gmLOINC.split_LOINCDBTXT(input_fname = os.path.join(loinc_dir, 'LOINCDB.TXT'))
 
 	wx.EndBusyCursor()
 
@@ -57,13 +56,9 @@ def update_loinc_reference_data():
 
 	wx.BeginBusyCursor()
 
+	# import data
 	if gmLOINC.loinc_import(data_fname = data_fname, license_fname = license_fname, conn = conn):
 		gmDispatcher.send(signal = 'statustext', msg = _('Successfully imported LOINC reference data.'))
-		try:
-			os.remove(data_fname)
-			os.remove(license_fname)
-		except OSError:
-			_log.error('unable to remove [%s] or [%s]', data_fname, license_fname)
 	else:
 		gmDispatcher.send(signal = 'statustext', msg = _('Importing LOINC reference data failed.'), beep = True)
 
@@ -138,8 +133,6 @@ def plot_measurements(parent=None, tests=None):
 	script = template.instantiate()
 	script.data_filename = fname_data
 	script.generate_output(format = 'wxp') 		# Gnuplot output terminal
-#	if cleanup:
-#		script.cleanup()
 
 #================================================================
 #from Gnumed.wxGladeWidgets import wxgPrimaryCareVitalsInputPnl
@@ -1723,6 +1716,9 @@ limit 50"""
 		lab = gmPathLab.create_test_org(name = self.GetValue().strip())
 		self.SetText(value = lab['internal_name'], data = lab['pk'])
 		return
+	#------------------------------------------------------------
+	def _data2instance(self):
+		return gmPathLab.cTestOrg(aPK_obj = self.data)
 #----------------------------------------------------------------
 from Gnumed.wxGladeWidgets import wxgMeasurementTypeEAPnl
 
@@ -1977,97 +1973,135 @@ limit 50"""
 		)
 		self._TCTRL_comment_org.SetValue(gmTools.coalesce(self.data['comment_org'], u''))
 #================================================================
+_SQL_units_from_test_results = u"""
+	-- via clin.v_test_results.pk_type (for types already used in results)
+	SELECT
+		val_unit as data,
+		val_unit || ' (' || name_tt || ')' as unit,
+		1 as rank
+	FROM
+		clin.v_test_results
+	WHERE
+		(
+			val_unit %(fragment_condition)s
+				OR
+		conversion_unit %(fragment_condition)s
+			)
+		%(ctxt_type_pk)s
+		%(ctxt_test_name)s
+"""
+
+_SQL_units_from_test_types = u"""
+	-- via clin.test_type (for types not yet used in results)
+	SELECT
+		conversion_unit as data,
+		conversion_unit || ' (' || name || ')' as unit,
+		2 as rank
+	FROM
+		clin.test_type
+	WHERE
+		conversion_unit %(fragment_condition)s
+		%(ctxt_ctt)s
+"""
+
+_SQL_units_from_loinc_ipcc = u"""
+	-- via ref.loinc.ipcc_units
+	SELECT
+		ipcc_units as data,
+		ipcc_units || ' (' || term || ')' as unit,
+		3 as rank
+	FROM
+		ref.loinc
+	WHERE
+		ipcc_units %(fragment_condition)s
+		%(ctxt_loinc)s
+		%(ctxt_loinc_term)s
+"""
+
+_SQL_units_from_loinc_submitted = u"""
+	-- via ref.loinc.submitted_units
+	SELECT
+		submitted_units as data,
+		submitted_units || ' (' || term || ')'  as unit,
+		3 as rank
+	FROM
+		ref.loinc
+	WHERE
+		submitted_units %(fragment_condition)s
+		%(ctxt_loinc)s
+		%(ctxt_loinc_term)s
+"""
+
+_SQL_units_from_loinc_example = u"""
+	-- via ref.loinc.example_units
+	SELECT
+		example_units as data,
+		example_units || ' (' || term || ')'  as unit,
+		3 as rank
+	FROM
+		ref.loinc
+	WHERE
+		example_units %(fragment_condition)s
+		%(ctxt_loinc)s
+		%(ctxt_loinc_term)s
+"""
+
+_SQL_units_from_atc = u"""
+	-- via rev.atc.unit
+	SELECT
+		unit AS data,
+		unit AS unit,
+		1 AS rank
+	FROM
+		ref.atc
+	WHERE
+		unit IS NOT NULL
+			AND
+		unit %(fragment_condition)s
+"""
+
+_SQL_units_from_consumable_substance = u"""
+	-- via ref.consumable_substance.unit
+	SELECT
+		unit AS data,
+		unit AS unit,
+		1 AS rank
+	FROM
+		ref.consumable_substance
+	WHERE
+		unit %(fragment_condition)s
+		%(ctxt_substance)s
+"""
+#================================================================
 class cUnitPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
 
 		query = u"""
-
 SELECT DISTINCT ON (data) data, unit FROM (
 
 	SELECT rank, data, unit FROM (
-
-		(
-		-- via clin.v_test_results.pk_type (for types already used in results)
-		SELECT
-			val_unit as data,
-			val_unit || ' (' || name_tt || ')' as unit,
-			1 as rank
-		FROM
-			clin.v_test_results
-		WHERE
-			(
-				val_unit %(fragment_condition)s
-					OR
-				conversion_unit %(fragment_condition)s
-			)
-			%(ctxt_type_pk)s
-			%(ctxt_test_name)s
-
-		) UNION ALL (
-
-		-- via clin.test_type (for types not yet used in results)
-		SELECT
-			conversion_unit as data,
-			conversion_unit || ' (' || name || ')' as unit,
-			2 as rank
-		FROM
-			clin.test_type
-		WHERE
-			conversion_unit %(fragment_condition)s
-			%(ctxt_ctt)s
-
-		) UNION ALL (
-
-		-- via ref.loinc.ipcc_units
-		SELECT
-			ipcc_units as data,
-			ipcc_units || ' (' || term || ')' as unit,
-			3 as rank
-		FROM
-			ref.loinc
-		WHERE
-			ipcc_units %(fragment_condition)s
-			%(ctxt_loinc)s
-			%(ctxt_loinc_term)s
-
-		) UNION ALL (
-
-		-- via ref.loinc.submitted_units
-		SELECT
-			submitted_units as data,
-			submitted_units || ' (' || term || ')'  as unit,
-			3 as rank
-		FROM
-			ref.loinc
-		WHERE
-			submitted_units %(fragment_condition)s
-			%(ctxt_loinc)s
-			%(ctxt_loinc_term)s
-
-		) UNION ALL (
-
-		-- via ref.loinc.example_units
-		SELECT
-			example_units as data,
-			example_units || ' (' || term || ')'  as unit,
-			3 as rank
-		FROM
-			ref.loinc
-		WHERE
-			example_units %(fragment_condition)s
-			%(ctxt_loinc)s
-			%(ctxt_loinc_term)s
-		)
-
+		(%s) UNION ALL
+		(%s) UNION ALL
+		(%s) UNION ALL
+		(%s) UNION ALL
+		(%s) UNION ALL
+		(%s) UNION ALL
+		(%s)
 	) as all_matching_units
-
 	WHERE data IS NOT NULL
 	ORDER BY rank
 
-) as ranked_matching_units
-
-LIMIT 50"""
+) AS ranked_matching_units
+LIMIT 50""" % (
+			_SQL_units_from_test_results,
+			_SQL_units_from_test_types,
+			_SQL_units_from_loinc_ipcc,
+			_SQL_units_from_loinc_submitted,
+			_SQL_units_from_loinc_example,
+			_SQL_units_from_atc,
+			_SQL_units_from_consumable_substance
+		)
 
 		ctxt = {
 			'ctxt_type_pk': {
@@ -2089,6 +2123,10 @@ LIMIT 50"""
 			'ctxt_loinc_term': {
 				'where_part': u'AND term ~* %(test_name)s',
 				'placeholder': u'test_name'
+			},
+			'ctxt_substance': {
+				'where_part': u'AND description ~* %(substance)s',
+				'placeholder': u'substance'
 			}
 		}
 
@@ -2101,7 +2139,7 @@ LIMIT 50"""
 			**kwargs
 		)
 		self.matcher = mp
-		self.SetToolTipString(_('Select the unit of the test result.'))
+		self.SetToolTipString(_('Select the desired unit for the amount or measurement.'))
 		self.selection_only = False
 		self.phrase_separators = u'[;|]+'
 #================================================================
@@ -2230,7 +2268,7 @@ class cMeasurementOrgEAPnl(wxgMeasurementOrgEAPnl.wxgMeasurementOrgEAPnl, gmEdit
 	#----------------------------------------------------------------
 	def _save_as_new(self):
 		# save the data as a new instance
-		data = self._PRW_name.GetData(can_create = True)
+		data = self._PRW_name.GetData(can_create = True, as_instance = True)
 
 		data['contact'] = self._TCTRL_contact.GetValue().strip()
 		data['comment'] = self._TCTRL_comment.GetValue().strip()
@@ -2314,7 +2352,7 @@ if __name__ == '__main__':
 
 	#------------------------------------------------------------
 	def test_grid():
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		app = wx.PyWidgetTester(size = (500, 300))
 		lab_grid = cMeasurementsGrid(parent = app.frame, id = -1)
 		lab_grid.patient = pat
@@ -2322,7 +2360,7 @@ if __name__ == '__main__':
 		app.MainLoop()
 	#------------------------------------------------------------
 	def test_test_ea_pnl():
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		gmPatSearchWidgets.set_active_patient(patient=pat)
 		app = wx.PyWidgetTester(size = (500, 300))
 		ea = cMeasurementEditAreaPnl(parent = app.frame, id = -1)
